// 2PatientView.cpp : implementation of the C2PatientView class
//

#include "stdafx.h"
#include "2Patient.h"

#include "2PatientDoc.h"
#include "2PatientView.h"

#include "Engine/GameModel.h"
#include "Engine/GameCardStack.h"
#include "Engine/GameBoard.h"
#include "Engine/GamePlayer.h"
#include "GameCardRenderer.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

enum { MAX_WIDTH = 1024, MAX_HEIGHT = 768 };

/////////////////////////////////////////////////////////////////////////////
// C2PatientView

IMPLEMENT_DYNCREATE(C2PatientView, CView)

BEGIN_MESSAGE_MAP(C2PatientView, CView)
	//{{AFX_MSG_MAP(C2PatientView)
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// C2PatientView construction/destruction

C2PatientView::C2PatientView() : 
	iCanvasDC(NULL), iCanvasBitmap(NULL), iMovingCards(false), iRedrawWindow(false)
{
}

C2PatientView::~C2PatientView()
{
	if (iCanvasDC != NULL)
		iCanvasDC->SelectObject(iOldCanvasDCBitmap);
	delete iCanvasBitmap;
	delete iCanvasDC;
}

BOOL C2PatientView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// C2PatientView drawing

void C2PatientView::OnDraw(CDC* pDC)
{
	C2PatientDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CGameModel& model = pDoc->Model();
	CGameCardRenderer& renderer = pDoc->GameCardRenderer();

	if (iCanvasDC == NULL)
	{
		iCanvasDC = new CDC;
		iCanvasDC->CreateCompatibleDC(pDC);
		iCanvasBitmap = new CBitmap;
		iCanvasBitmap->CreateCompatibleBitmap(pDC, MAX_WIDTH, MAX_HEIGHT);
		iOldCanvasDCBitmap = iCanvasDC->SelectObject(iCanvasBitmap);

		model.GameBoard().SetGameBoardRenderer(this);
	}

	CRect rectClient;

	if (!iRedrawWindow)
		GetClientRect(rectClient);
	else
		rectClient = iRedrawRect;

	CRect rect(rectClient);
	if (rect.right >= MAX_WIDTH)
		rect.right = MAX_WIDTH - 1;
	if (rect.bottom >= MAX_HEIGHT)
		rect.bottom = MAX_HEIGHT - 1;

	CBrush boardBrush;
	boardBrush.CreateSolidBrush(RGB(0,128,0));
	iCanvasDC->FillRect(rect, &boardBrush);

	// rendering optimzation (todo), create render stack method which
	// ensures that cards under other cards are not drawn

	CGameCardStack *stackToDrawLast = 0;
	if (model.GameState() == EGameStatePlayer1MakingMove ||
		model.GameState() == EGameStatePlayer2MakingMove)
	{
		stackToDrawLast = &model.GameBoard().CardStackMove();
	}

	for (int i = 0; i < model.StackCount(); ++i)
	{
		CGameCardStack& stack = static_cast<CGameCardStack&>(model.CardStack(i));
		if (&stack == stackToDrawLast)
			continue;
		renderer.DrawCardStack(*iCanvasDC, rect, stack);
	}

	if (stackToDrawLast)
	{
		renderer.DrawCardStack(*iCanvasDC, rect, *stackToDrawLast);
	}

	pDC->BitBlt(rect.left, rect.top, rect.Width(), rect.Height(), iCanvasDC, 
		rect.left, rect.top, SRCCOPY);

	pDC->ExcludeClipRect(CRect(0, 0, MAX_WIDTH, MAX_HEIGHT));
	pDC->FillRect(rectClient, &boardBrush);

	boardBrush.DeleteObject();
}

void C2PatientView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
	if (!lHint)
		UpdateSpacialInformation();

	CRect rect;
	GetClientRect(rect);
	iRedrawWindow = true;
	iRedrawRect = rect;
	RedrawWindow(&rect, NULL);
	iRedrawWindow = false;
}

void C2PatientView::RenderGameBoard(const SSpacialInformation& aRect)
{
	CRect rect(aRect.iX, aRect.iY, 
		aRect.iX + aRect.iWidth - 1, aRect.iY + aRect.iHeight - 1);
	iRedrawWindow = true;
	iRedrawRect = rect;
	RedrawWindow(&rect, NULL);
	iRedrawWindow = false;
}

void C2PatientView::UpdateSpacialInformation()
{
	CRect rect;
	GetClientRect(rect);

	C2PatientDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (rect.right > MAX_WIDTH - 1)
		rect.right = MAX_WIDTH - 1;
	if (rect.bottom > MAX_HEIGHT - 1)
		rect.bottom = MAX_HEIGHT - 1;

	ISpacialInformation& gameSpace = static_cast<ISpacialInformation&>(pDoc->Model().GameBoard());
	gameSpace.SetSpacialInformation(0, 0, rect.right, rect.bottom);
}

/////////////////////////////////////////////////////////////////////////////
// C2PatientView diagnostics

#ifdef _DEBUG
void C2PatientView::AssertValid() const
{
	CView::AssertValid();
}

void C2PatientView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

C2PatientDoc* C2PatientView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(C2PatientDoc)));
	return (C2PatientDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// C2PatientView message handlers

void C2PatientView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);

	UpdateSpacialInformation();
	Invalidate(FALSE);	
}

BOOL C2PatientView::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

void C2PatientView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CView::OnMouseMove(nFlags, point);

	C2PatientDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (pDoc->Model().CurrentPlayer().ComputerPlayer())
		return;

	if (pDoc->Model().GameState() != EGameStatePlayer1MakingMove &&
		pDoc->Model().GameState() != EGameStatePlayer2MakingMove)
	{
		return;
	}

	int x1, y1, x2, y2;
	pDoc->Model().GameBoard().SelectionBoundingRect(x1, y1, x2, y2, true);
	pDoc->Model().GameBoard().MoveSelection(point.x, point.y);
	pDoc->Model().GameBoard().SelectionBoundingRect(x1, y1, x2, y2, false);

	CRect rect(x1, y1, x2+1, y2+1);
	iRedrawWindow = true;
	iRedrawRect = rect;
	RedrawWindow(&rect, NULL);
	iRedrawWindow = false;
}

void C2PatientView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	C2PatientDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (pDoc->Model().CurrentPlayer().ComputerPlayer())
		return;

	if (pDoc->Model().GameState() == EGameStatePlayer1MakingMove ||
		pDoc->Model().GameState() == EGameStatePlayer2MakingMove)
	{
		OnLButtonUp(nFlags, point);
		return;
	}

	if (pDoc->Model().GameBoard().StartMove(point.x, point.y))
	{
		ShowCursor(FALSE);
		SetCapture();
	}
}

void C2PatientView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	C2PatientDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (pDoc->Model().CurrentPlayer().ComputerPlayer())
		return;

	if (GetCapture() == this)
	{
		ReleaseCapture();
		ShowCursor(TRUE);	
	}

	if (pDoc->Model().GameState() == EGameStatePlayer1MakingMove ||
		pDoc->Model().GameState() == EGameStatePlayer2MakingMove)
	{
		pDoc->Model().GameBoard().EndMove(point.x, point.y);
	}

	UpdateSpacialInformation();
	Invalidate(FALSE);
}

void C2PatientView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	C2PatientDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if (pDoc->Model().CurrentPlayer().ComputerPlayer())
		return;

	if (pDoc->Model().GameBoard().StartMove(point.x, point.y, true))
	{
		pDoc->Model().GameBoard().EndMove(point.x, point.y, true);
	}
}
