#if !defined(GameBoard_H_Included)
#define GameBoard_H_Included

// GameBoard.h : header file
//

#include "GameCommon.h"
#include "SpacialInformation.h"
#include "GameState.h"

class CGameModel;
class CGameCardStack;
class CGameCard;
struct IGameBoardRenderer;

class CGameBoard : public CGameObject, public SSpacialInformation, private IGameStateObserver
{
public:
	enum EBoardLayout
	{
		ELayoutTower
	};

	struct SCardImageSize : public SSpacialInformation {};

// Construction
public:
	CGameBoard(CGameModel& aModel, EBoardLayout aLayout = ELayoutTower);
	virtual ~CGameBoard();

// Attributes
private:
	CGameModel& iGameModel;
	EBoardLayout iBoardLayout;
	SCardImageSize iCardImageSize;
	CGameCardStack* iCardStackMove;
	CGameCard* iCardMove;
	int iStartMoveOffsetX, iStartMoveOffsetY;
	bool iFaceDownAtStartMove;
	int iNumberOfInStacks;
	IGameBoardRenderer* iGameBoardRenderer;

// Operations
public:
	bool StartMove(int aX, int aY, bool aOutPass = false);
	bool StartMove(CGameCardStack& aSrcStack, CGameCard& aGameCard, bool aOutPass = false);
	bool EndMove(int aX, int aY, bool aOutPass = false);
	bool EndMove(CGameCardStack& aTargetStack, bool aOutPass = false);
	bool CardStackAtLocation(int aX, int aY, CGameCardStack*& aGameCardStack, CGameCard*& aGameCard) const;
	bool CardAtLocation(int aX, int aY, CGameCardStack*& aGameCardStack, CGameCard*& aGameCard) const;
	SCardImageSize& CardImageSize() { return iCardImageSize; }
	CGameCardStack& CardStackMove() { return *iCardStackMove; }
	CGameCard& CardMove() { return *iCardMove; }
	void SelectionBoundingRect(int& aX1, int& aY1, int& aX2, int& aY2, bool aInit);
	void MoveSelection(int aX, int aY);
	void AnimateCardMove(CGameCardStack& aTarget);

	void SetGameBoardRenderer(IGameBoardRenderer* aRenderer) { iGameBoardRenderer = aRenderer; }


// Implementation
private:
// from IGameObject
#ifdef OS_SYMBIAN
	virtual void ConstructL();
#else // OS_WINDOWS
	virtual bool InitResources();
#endif
	virtual void NewGame(int aNumberOfPlayers);
	virtual bool SerializeIn(const unsigned char*& aBufferPtr, int& aBufferBytesLeft);
	virtual bool SerializeOut(unsigned char*& aBufferPtr, int& aBufferBytesLeft) const;
// from ISpacialInformation
	virtual void SpacialInformation(int& aXPosition, int& aYPosition, int& aWidth, int& aHeight) const; 
	virtual void SetSpacialInformation(int aXPosition, int aYPosition, int aWidth, int aHeight);
// from IGameStateObserver
	virtual void OnSetGameState(CGameModel& /*aModel*/, EGameState /*aNewState*/) {}
	virtual void OnGameStateChanged(CGameModel& aModel, EGameState aOldState, EGameState aNewState);
};

/////////////////////////////////////////////////////////////////////////////

#endif // !defined(GameBoard_H_Included)
