#if !defined(GameCard_H_Included)
#define GameCard_H_Included

// GameCard.h : header file
//

#include "GameCommon.h"
#include "SpacialInformation.h"

class CGameCard : public CGameObject, public SSpacialInformation
{
// API
public:
	enum ECardValue
	{
		EAce,
		ETwo,
		EThree,
		EFour,
		EFive,
		ESix,
		ESeven,
		EEight,
		ENine,
		ETen,
		EJack,
		EQueen,
		EKing,
	};
	enum ECardSuit 
	{	
		ESpades,
		EClubs,
		EHearts,
		EDiamonds,
	};
	enum ECardColour
	{
		EBlack,
		ERed,
	};

// Construction
public:
	CGameCard(ECardValue aValue, ECardSuit aSuit);
	virtual ~CGameCard();

// Attributes
private:
	ECardValue iValue;
	ECardSuit iSuit;
	bool iFaceDown;
	bool iDealt;

// Operations
public:
	ECardValue Value() const { return iValue; }
	ECardSuit Suit() const { return iSuit; }
	ECardColour Colour() const { return iSuit == EClubs || iSuit == ESpades ? EBlack : ERed; }
	bool FaceDown() const { return iFaceDown; }
	bool Dealt() const { return iDealt; }

	void SetSuit(ECardSuit aSuit) { iSuit = aSuit; }
	void SetValue(ECardValue aValue) { iValue = aValue; }
	void SetFaceDown(bool aFaceDown) { iFaceDown = aFaceDown; }
	void SetDealt(bool aDealt) { iDealt = aDealt; }

	void Swap(CGameCard& aOtherCard);

// Implementation
private:
// from IGameObject
#ifdef OS_SYMBIAN
	virtual void ConstructL();
#else // OS_WINDOWS
	virtual bool InitResources();
#endif	
	virtual void NewGame(int aNumberOfPlayers);
	virtual bool SerializeIn(const unsigned char*& aBufferPtr, int& aBufferBytesLeft);
	virtual bool SerializeOut(unsigned char*& aBufferPtr, int& aBufferBytesLeft) const;
// from ISpacialInformation
	virtual void SpacialInformation(int& aXPosition, int& aYPosition, int& aWidth, int& aHeight) const; 
	virtual void SetSpacialInformation(int aXPosition, int aYPosition, int aWidth, int aHeight);
};

/////////////////////////////////////////////////////////////////////////////

#endif // !defined(GameCard_H_Included)
