#if !defined(GameCardStack_H_Included)
#define GameCardStack_H_Included

// GameCardStack.h : header file
//

#include "GameCommon.h"
#include "GameCard.h"
#include "SpacialInformation.h"
#include "GameState.h"

class CGameCardStack : public CGameObject, public SSpacialInformation
{
// Primitives
public:
	enum EConstants 
	{ 
		ENumberOfCards = 52, 
		ENumberOfSuits = 4,
		ENumberOfValues = 13,
	};
	enum EType
	{
		EDeck, // one of these, this class owns the card objects
		EOutStack, // four of these, this class does not own the card objects
		EInStack, // four of these, this class does not own the card objects
		EPlayer1InHand, // one of these, this class does not own the card objects
		EPlayer1OutHand, // one of these, this class does not own the card objects
		EPlayer2OutHand, // one of these, this class does not own the card objects
		EPlayer2InHand // one of these, this class does not own the card objects
	};
	
// Construction
public:
	CGameCardStack(CGameModel& aModel, EType aType);
	virtual ~CGameCardStack();

// Attributes
protected:
	CGameModel& iModel;
	EType iType;
	CGameCard* iCards[ENumberOfCards];
	int iCardCount;

// Operations
public: // play API
	void Shuffle();
	EType Type() const { return iType; }
	CGameCard& Card(int aIndex) { return *(iCards[aIndex]); }
	const CGameCard& Card(int aIndex) const { return *(iCards[aIndex]); }
	CGameCard& TopCard() const { return *(iCards[iCardCount-1]); }
	int CardCount() const { return iCardCount; }
	int CardIndex(const CGameCard& aCard) const;
	bool CanMoveCard(EGameState aState, const CGameCard& aGameCard, const CGameCardStack& aTargetStack) const;
	void MoveCard(EGameState aState, CGameCard& aGameCard, CGameCardStack& aTargetStack);

public:  // utility API
	void SetCardCount(int aCount) { iCardCount = aCount; }
	bool SetCard(int aIndex, CGameCard& aGameCard);
	void AddCard(CGameCard& aGameCard);
	void RemoveCard(CGameCard& aGameCard);

// Implementation
private:
// from IGameObject
#ifdef OS_SYMBIAN
	virtual void ConstructL();
#else // OS_WINDOWS
	virtual bool InitResources();
#endif	
	virtual void NewGame(int aNumberOfPlayers);
	virtual bool SerializeIn(const unsigned char*& aBufferPtr, int& aBufferBytesLeft);
	virtual bool SerializeOut(unsigned char*& aBufferPtr, int& aBufferBytesLeft) const;
// from ISpacialInformation
	virtual void SpacialInformation(int& aXPosition, int& aYPosition, int& aWidth, int& aHeight) const;
	virtual void SetSpacialInformation(int aXPosition, int aYPosition, int aWidth, int aHeight);
};

/////////////////////////////////////////////////////////////////////////////

#endif // !defined(GameCardStack_H_Included)
