// GameModel.cpp : implementation file
//

#include "GameModel.h"
#include "GameBoard.h"
#include "GameCardStack.h"
#include "GamePlayer.h"

CGameModel::CGameModel() : 
	CGameState(*this), iNumberOfInStacks(4), iNumberOfInStacksNextGame(4), iNumberOfPlayersNextGame(1),
	iAllowISOOS(true), iAllowISOOSNextGame(true), iCardSpeed(5)
{
}


CGameModel::~CGameModel()
{
	delete iDeck;

	for (int i = 0; i < 15; ++i)
		delete iCardStacks[i];

	delete iGamePlayerOne;
	delete iGamePlayerTwo;

	delete iGameBoard;
}

#ifdef OS_SYMBIAN
void CGameModel::ConstructL()
{
	iDeck = new (ELeave) CGameCardStack(*this, CGameCardStack::EDeck);

	for (int i = EInStack1; i <= EInStack7; ++i)
		iCardStacks[i] = new (ELeave) CGameCardStack(*this, CGameCardStack::EInStack);

	for (int i = EOutStack1; i <= EOutStack4; ++i)
		iCardStacks[i] = new (ELeave) CGameCardStack(*this, CGameCardStack::EOutStack);
	
	iCardStacks[EPlayer1InHand] = new (ELeave) CGameCardStack(*this, CGameCardStack::EPlayer1InHand);
	iCardStacks[EPlayer2InHand] = new (ELeave) CGameCardStack(*this, CGameCardStack::EPlayer2InHand);

	iCardStacks[EPlayer1OutHand] = new (ELeave) CGameCardStack(*this, CGameCardStack::EPlayer1OutHand);
	iCardStacks[EPlayer2OutHand] = new (ELeave) CGameCardStack(*this, CGameCardStack::EPlayer2OutHand);

	iGamePlayerOne = new (ELeave) CGamePlayer(*this, 1);
	iGamePlayerTwo = new (ELeave) CGamePlayer(*this, 2);

	iGameBoard = new (ELeave) CGameBoard(*this);

	static_cast<IGameObject*>(iDeck)->ConstructL();

	for (int i = 0; i < 15; ++i)
		static_cast<IGameObject*>(iCardStacks[i])->ConstructL();

	static_cast<IGameObject*>(iGamePlayerOne)->ConstructL();
	static_cast<IGameObject*>(iGamePlayerTwo)->ConstructL();

	static_cast<IGameObject*>(iGameBoard)->ConstructL();
}
#else // not OS_SYMBIAN
bool CGameModel::InitResources()
{
	iDeck = new CGameCardStack(*this, CGameCardStack::EDeck);

	for (int i = EInStack1; i <= EInStack7; ++i)
		iCardStacks[i] = new CGameCardStack(*this, CGameCardStack::EInStack);

	for (int i = EOutStack1; i <= EOutStack4; ++i)
		iCardStacks[i] = new CGameCardStack(*this, CGameCardStack::EOutStack);
	
	iCardStacks[EPlayer1InHand] = new CGameCardStack(*this, CGameCardStack::EPlayer1InHand);
	iCardStacks[EPlayer2InHand] = new CGameCardStack(*this, CGameCardStack::EPlayer2InHand);

	iCardStacks[EPlayer1OutHand] = new CGameCardStack(*this, CGameCardStack::EPlayer1OutHand);
	iCardStacks[EPlayer2OutHand] = new CGameCardStack(*this, CGameCardStack::EPlayer2OutHand);

	iGamePlayerOne = new CGamePlayer(*this, 1);
	iGamePlayerTwo = new CGamePlayer(*this, 2);

	iGameBoard = new CGameBoard(*this);

	static_cast<IGameObject*>(iDeck)->InitResources();

	for (int i = 0; i < 15; ++i)
		static_cast<IGameObject*>(iCardStacks[i])->InitResources();

	static_cast<IGameObject*>(iGamePlayerOne)->InitResources();
	static_cast<IGameObject*>(iGamePlayerTwo)->InitResources();

	static_cast<IGameObject*>(iGameBoard)->InitResources();

	return true;
}
#endif


void CGameModel::NewGame(int aNumberOfPlayers)
{
	SetGameState(EGameStateDealing);

	iNumberOfInStacks = iNumberOfInStacksNextGame;
	iAllowISOOS = iAllowISOOSNextGame;

	static_cast<IGameObject*>(iDeck)->NewGame(aNumberOfPlayers);

	for (int i = 0; i < 15; ++i)
		static_cast<IGameObject*>(iCardStacks[i])->NewGame(aNumberOfPlayers);

	static_cast<IGameObject*>(iGamePlayerOne)->NewGame(aNumberOfPlayers);
	static_cast<IGameObject*>(iGamePlayerTwo)->NewGame(aNumberOfPlayers);

	static_cast<IGameObject*>(iGameBoard)->NewGame(aNumberOfPlayers);

	SetGameState(EGameStatePlayer1ToMove);
}

bool CGameModel::SerializeIn(const unsigned char*& aBufferPtr, int& aBufferBytesLeft)
{
	if (aBufferBytesLeft < 4)
		return false;

	int stateLow = *aBufferPtr++;
	int stateHi = *aBufferPtr++;
	EGameState state = static_cast<EGameState>(stateLow % 256 + stateHi * 256);
	--aBufferBytesLeft;
	--aBufferBytesLeft;
	iNumberOfInStacks = (*aBufferPtr++);
	--aBufferBytesLeft;
	iAllowISOOS = (*aBufferPtr++ == 1 ? true : false);
	--aBufferBytesLeft;

	iDeck->SetCardCount(0);
	int deckIndex = 0;

	for (int i = 0; i < 15; ++i)
	{
		if (aBufferBytesLeft < 2)
			return false;
 
		// prefetch stack info so we can allocate cards from deck
		CGameCardStack::EType type = static_cast<CGameCardStack::EType>(*aBufferPtr++);
		--aBufferBytesLeft;
		int cardCount = (*aBufferPtr++);
		--aBufferBytesLeft;

		aBufferPtr -= 2;
		aBufferBytesLeft += 2;

		iCardStacks[i]->SetCardCount(cardCount);

		for (int j = 0; j < cardCount; ++j)
		{
			CGameCard& card = iDeck->Card(deckIndex++);
			card.SetDealt(true);
			iCardStacks[i]->SetCard(j, card);
		}

		static_cast<ISerializable*>(iCardStacks[i])->SerializeIn(aBufferPtr, aBufferBytesLeft);
	}

	static_cast<CGameCardStack*>(iDeck)->SetCardCount(0);

	static_cast<ISerializable*>(iGamePlayerOne)->SerializeIn(aBufferPtr, aBufferBytesLeft);
	static_cast<ISerializable*>(iGamePlayerTwo)->SerializeIn(aBufferPtr, aBufferBytesLeft);

	static_cast<ISerializable*>(iGameBoard)->SerializeIn(aBufferPtr, aBufferBytesLeft);

	SetGameState(state);

	return true;
}

bool CGameModel::SerializeOut(unsigned char*& aBufferPtr, int& aBufferBytesLeft) const
{
	if (aBufferBytesLeft < 4)
		return false;

	(*aBufferPtr++) = iGameState % 256;
	--aBufferBytesLeft;
	(*aBufferPtr++) = iGameState / 256;
	--aBufferBytesLeft;
	(*aBufferPtr++) = iNumberOfInStacks;
	--aBufferBytesLeft;
	(*aBufferPtr++) = static_cast<unsigned char>(iAllowISOOS ? 1 : 0);
	--aBufferBytesLeft;
	
	for (int i = 0; i < 15; ++i)
	{
		static_cast<ISerializable*>(iCardStacks[i])->SerializeOut(aBufferPtr, aBufferBytesLeft);
	}

	static_cast<ISerializable*>(iGamePlayerOne)->SerializeOut(aBufferPtr, aBufferBytesLeft);
	static_cast<ISerializable*>(iGamePlayerTwo)->SerializeOut(aBufferPtr, aBufferBytesLeft);

	static_cast<ISerializable*>(iGameBoard)->SerializeOut(aBufferPtr, aBufferBytesLeft);

	return true;
}

void CGameModel::AddMove(int /*aSourceGameCardStackIndex*/, int /*aDestinationGameCardStackIndex*/)
{
}

void CGameModel::UndoMove()
{
	// todo
}

void CGameModel::RedoMove()
{
	// todo
}

bool CGameModel::CanUndoMove()
{
	return true;
}

bool CGameModel::CanRedoMove()
{
	return true;
}
