#if !defined(GameModel_h_Included)
#define GameModel_h_Included

// GameModel.h : header file
//


#include "GameObject.h"
#include "GameState.h"

class CGameCardStack;
class CGameBoard;
class CGamePlayer;

class CGameModel : public CGameState
{
// API
public:
	struct SMove
	{
		EGameState iGameState;
		int iSourceGameCardStackIndex;
		int iDestinationGameCardStackIndex;
	};

// Construction
public:
	CGameModel();
	virtual ~CGameModel();

// Attributes
public:
	enum EStackIndex
	{
		EPlayer1InHand,
		EPlayer2InHand,
		EPlayer1OutHand,
		EPlayer2OutHand,
		EOutStack1,
		EOutStack2,
		EOutStack3,
		EOutStack4,
		EInStack1,
		EInStack2,
		EInStack3,
		EInStack4,
		EInStack5,
		EInStack6,
		EInStack7
	};
private:
	CGameCardStack* iDeck;
	CGameCardStack* iCardStacks[15];
	CGameBoard* iGameBoard;
	CGamePlayer* iGamePlayerOne;
	CGamePlayer* iGamePlayerTwo;
	int iNumberOfInStacks;
	int iNumberOfInStacksNextGame;
	int iNumberOfPlayersNextGame;
	bool iAllowISOOS;
	bool iAllowISOOSNextGame;
	int iCardSpeed;

// API
public:
	CGameCardStack& CardDeck() { return *iDeck; }
	CGameCardStack& CardInStack(int nStack) { return *iCardStacks[EInStack1 + nStack]; }
	CGameCardStack& CardOutStack(int nStack) { return *iCardStacks[EOutStack1 + nStack]; }
	CGameCardStack& PlayerInHand(int nStack) { return *iCardStacks[EPlayer1InHand + nStack]; }
	CGameCardStack& PlayerOutHand(int nStack) { return *iCardStacks[EPlayer1OutHand + nStack]; }
	CGameBoard& GameBoard() { return *iGameBoard; }
	CGamePlayer& GamePlayerOne() { return *iGamePlayerOne; }
	CGamePlayer& GamePlayerTwo() { return *iGamePlayerTwo; }
	CGamePlayer& CurrentPlayer() { return iGameState & EGameStatePlayer2 ? GamePlayerTwo() : GamePlayerOne(); }
	int NumberOfInStacks() const { return iNumberOfInStacks; }
	void SetNumberOfInStacks(int aNumberOfStacks) { iNumberOfInStacks = aNumberOfStacks; }
	int NumberOfInStacksNextGame() const { return iNumberOfInStacksNextGame; }
	void SetNumberOfInStacksNextGame(int aNumberOfStacks) { iNumberOfInStacksNextGame = aNumberOfStacks; }
	int NumberOfPlayersNextGame() const { return iNumberOfPlayersNextGame; }
	void SetNumberOfPlayersNextGame(int aNumberOfPlayers) { iNumberOfPlayersNextGame = aNumberOfPlayers; }
	bool AllowISOOS() const { return iAllowISOOS; }
	bool AllowISOOSNextGame() const { return iAllowISOOSNextGame; }
	void SetAllowISOOSNextGame(bool aAllowISOOS) { iAllowISOOSNextGame = aAllowISOOS; }
	int CardSpeed() const { return iCardSpeed; }
	void SetCardSpeed(int nSpeed) { iCardSpeed = nSpeed; }
	int StackCount() const { return 8 + iNumberOfInStacks; }
	CGameCardStack& CardStack(int nStack) { return *iCardStacks[nStack]; }
	
	void AddMove(int aSourceGameCardStackIndex, int aDestinationGameCardStackIndex);
	void UndoMove();
	void RedoMove();
	bool CanUndoMove();
	bool CanRedoMove();

// Implementation
private:
	// from IGameObject
#ifdef OS_SYMBIAN
	virtual void ConstructL();
#else // OS_WINDOWS
	virtual bool InitResources();
#endif	
	virtual void NewGame(int aNumberOfPlayers); // 0, 1 or 2 players
	virtual bool SerializeIn(const unsigned char*& aBufferPtr, int& aBufferBytesLeft);
	virtual bool SerializeOut(unsigned char*& aBufferPtr, int& aBufferBytesLeft) const;
};

/////////////////////////////////////////////////////////////////////////////

#endif // !defined(GameModel_h_Included)
