// GamePlayer.cpp : implementation file
//

#include "GamePlayer.h"

#include "GameModel.h"
#include "GameCardStack.h"
#include "GameBoard.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CGamePlayer

CGamePlayer::CGamePlayer(CGameModel& aModel, int aPlayerNumber) :
	iGameModel(aModel), iPlayerNumber(aPlayerNumber), 
	iComputerPlayer(false), 
	iTrySrcStackCardIndex(0), iTryTargetStackIndex(0), iTrySrcStackIndex(0), 
	iIncrTargetStackIndex(false), iAnotherRound(false)
{
}

CGamePlayer::~CGamePlayer()
{
}

void CGamePlayer::NewGame(int aNumberOfPlayers)
{
	switch (aNumberOfPlayers)
	{
	case 0:
		iComputerPlayer = true;
		break;
	case 1:
		if (iPlayerNumber == 2)
			iComputerPlayer = true;
		else
			iComputerPlayer = false;
		break;
	case 2:
		iComputerPlayer = false;
		break;
	}
}

#ifdef OS_SYMBIAN
void CGamePlayer::ConstructL()
{
	iGameModel.AddObserver(*this);
}
#else // not OS_SYMBIAN
bool CGamePlayer::InitResources()
{
	iGameModel.AddObserver(*this);
	return true;
}
#endif

bool CGamePlayer::SerializeIn(const unsigned char*& aBufferPtr, int& aBufferBytesLeft)
{
	if (aBufferBytesLeft < 4)
		return false;

	iPlayerNumber = *aBufferPtr++;
	--aBufferBytesLeft;
	iComputerPlayer = (*aBufferPtr++ == 1 ? true : false);
	--aBufferBytesLeft;

	return true;
}

bool CGamePlayer::SerializeOut(unsigned char*& aBufferPtr, int& aBufferBytesLeft) const
{
	if (aBufferBytesLeft < 4)
		return false;

	*aBufferPtr++ = iPlayerNumber % 256;
	--aBufferBytesLeft;
	*aBufferPtr++ = (iComputerPlayer ? 1 : 0);
	--aBufferBytesLeft;
	
	return true;
}

#include "logging.h"

void CGamePlayer::OnGameStateChanged(CGameModel& aModel, EGameState aOldState, EGameState aNewState)
{
	if (!iComputerPlayer)
	{
		return;
	}

	if (aModel.GameState() & EGameStateWon)
		return;

	if (&aModel.CurrentPlayer() != this)
	{
		iTrySrcStackIndex = 0;
		iTrySrcStackCardIndex = 0;
		iTryTargetStackIndex = 0;
		iIncrTargetStackIndex = false;
		return;
	}

	iAnotherRound = false;

	switch (aNewState)
	{
	case EGameStatePlayer1ToMove:
	case EGameStatePlayer2ToMove:
		{
			while(!(aModel.GameState() & EGameStateWon))
			{
				while(!(aModel.GameState() & EGameStateWon))
				{
					if (&aModel.CardStack(iTryTargetStackIndex) == &aModel.PlayerOutHand(iPlayerNumber-1))
						++iTryTargetStackIndex;
					else if (aModel.CardStack(iTryTargetStackIndex).Type() == CGameCardStack::EPlayer1InHand ||
						aModel.CardStack(iTryTargetStackIndex).Type() == CGameCardStack::EPlayer2InHand)
						++iTryTargetStackIndex;
					else
						break;
				}

				int trySrcStackIndex = iTrySrcStackIndex;
				int tryTargetStackIndex = iTryTargetStackIndex;

				if (trySrcStackIndex == CGameModel::EPlayer1OutHand && iPlayerNumber == 2)
					trySrcStackIndex = CGameModel::EPlayer2OutHand;
				else if (trySrcStackIndex == CGameModel::EPlayer2OutHand && iPlayerNumber == 1)
					trySrcStackIndex = CGameModel::EPlayer1OutHand;
	

				CGameCardStack& srcStack = aModel.CardStack(trySrcStackIndex);
				CGameCardStack& targetStack = aModel.CardStack(tryTargetStackIndex);
				int maxCardIndex = srcStack.CardCount() - 1;

				if (srcStack.Type() != CGameCardStack::EInStack)
				{
					iTrySrcStackCardIndex = maxCardIndex;
				}

				if (iTrySrcStackCardIndex > maxCardIndex)
				{
					iTrySrcStackCardIndex = maxCardIndex;
				}

				if (maxCardIndex >= 0  && 
					aModel.GameBoard().StartMove(srcStack, srcStack.Card(iTrySrcStackCardIndex)) &&
					aModel.GameBoard().EndMove(targetStack))
				{

					if ((srcStack.Type() == CGameCardStack::EPlayer1InHand &&
						targetStack.Type() == CGameCardStack::EPlayer1OutHand) ||
						(srcStack.Type() == CGameCardStack::EPlayer2InHand &&
						targetStack.Type() == CGameCardStack::EPlayer2OutHand))
						iAnotherRound = false;
					else
						iAnotherRound = true;
					if (!(aModel.GameState() & EGameStateWon))
					{
						aModel.SetGameState(static_cast<EGameState>(aNewState & (EGameStateEitherPlayer | EGameStateThinking)));
						aModel.SetGameState(static_cast<EGameState>(aNewState & (EGameStateEitherPlayer | EGameStateToMove)));
					}
					return;
				}
				else
				{
					if (++iTrySrcStackCardIndex >= srcStack.CardCount())
					{
						iTrySrcStackCardIndex = 0;
						if (++iTrySrcStackIndex >= aModel.StackCount())
						{
							iTrySrcStackIndex = 0;
							if (++iTryTargetStackIndex >= aModel.StackCount())
							{
								iTryTargetStackIndex = 0;
								if (!iAnotherRound)
								{
									CGameCardStack& inHand = aModel.PlayerInHand(iPlayerNumber-1);
									for (int i = 0; i < aModel.NumberOfInStacks(); ++i)
									{
										if (aModel.CardInStack(i).CardCount() == 0)
										{
											iAnotherRound = true;
											break;
										}
									}
									if (!iAnotherRound)
									{
										if (inHand.CardCount())
										{
											aModel.GameBoard().StartMove(inHand, inHand.TopCard(), true);
											aModel.GameBoard().EndMove(inHand, true);
										}
										else if (aModel.PlayerOutHand(iPlayerNumber-1).CardCount() == 0)
										{
											aModel.SetGameState(static_cast<EGameState>((aModel.GameState() & EGameStateEitherPlayer) | EGameStateWon));
										}
										return;
									}
								}
								else
								{
									aModel.SetGameState(static_cast<EGameState>(aNewState & (EGameStateEitherPlayer | EGameStateThinking)));
									aModel.SetGameState(static_cast<EGameState>(aNewState & (EGameStateEitherPlayer | EGameStateToMove)));
									iAnotherRound = false;
									return;
								}
							}
						}
					}
				}
				aModel.SetGameState(static_cast<EGameState>(aNewState & (EGameStateEitherPlayer | EGameStateThinking)));
				aModel.SetGameState(static_cast<EGameState>(aNewState & (EGameStateEitherPlayer | EGameStateToMove)));
			}
		}
		break;
	case EGameStatePlayer1MakingMove:
	case EGameStatePlayer2MakingMove:
		break;
	default:
		break;
	}
}
