#if !defined(GameState_H_Included)
#define GameState_H_Included

// GameState.h : header file
//

#include "GameObject.h"

class CGameModel;

enum EGameState
{
	EGameStateUnknown			= 0x0000,
	EGameStateNone				= EGameStateUnknown, // a friendly alias
	EGameStateInvalid			= EGameStateUnknown, // a friendly alias
	EGameStateDealing			= 0x0001,
	EGameStateWon				= EGameStateDealing,
	EGameStateToMove			= 0x0002,
	EGameStateMakingMove		= 0x0004,
	EGameStateMadeMove			= 0x0008,
	EGameStateThinking			= 0x0010,
	EGameStatePlayer1			= 0x0100,
	EGameStatePlayer1Won		= 0x0101,
	EGameStatePlayer1ToMove		= 0x0102,
	EGameStatePlayer1MakingMove	= 0x0104,
	EGameStatePlayer1MadeMove	= 0x0108,
	EGameStatePlayer1Thinking	= 0x0110,
	EGameStatePlayer2			= 0x0200,
	EGameStatePlayer2Won		= 0x0201,
	EGameStatePlayer2ToMove		= 0x0202,
	EGameStatePlayer2MakingMove	= 0x0204,
	EGameStatePlayer2MadeMove	= 0x0208,
	EGameStatePlayer2Thinking	= 0x0210,
	EGameStateEitherPlayer		= 0x0300,
	EGameStateSerializing		= 0x0400,
	EGameStateGameOver			= 0x1000
};

struct IGameStateObserver
{
	virtual void OnSetGameState(CGameModel& aModel, EGameState aNewState) = 0;
	virtual void OnGameStateChanged(CGameModel& aModel, EGameState aOldState, EGameState aNewState) = 0;
};


class CGameState : public CGameObject
{
// Construction
protected:
	CGameState(CGameModel& aModel) : iModel(aModel), iNotifyObserversStateChanged(false), iObserverCount(0), iGameState(EGameStateDealing), iGameStatePrevious(EGameStateUnknown) {}

// Operations
public:
	EGameState GameState() const { return iGameState; }
	EGameState GameStatePrevious() const { return iGameStatePrevious; }
	void SetGameState(EGameState aNewState);
	void NotifyObserversSetState();
	bool NeedNotification() const; 
	void NotifyObserversStateChanged();
	void AddObserver(IGameStateObserver& aObserver);
	void RemoveObserver(IGameStateObserver& aObserver);
	
// Attributes
private:
	CGameModel& iModel;
	bool iNotifyObserversStateChanged;
	enum { MAX_OBSERVERS = 10 };
	IGameStateObserver* iObservers[MAX_OBSERVERS];
	int iObserverCount;
protected:
	EGameState iGameState;
	EGameState iGameStatePrevious;
};


/////////////////////////////////////////////////////////////////////////////

#endif // !defined(GameState_H_Included)
