#ifndef _LINKED_LIST_H_
#define _LINKED_LIST_H_

template<class T>
class list
{
private:
	struct Node
	{
		Node* iNext;
		Node* iPrev;
		T iData;
	};

public:
	class iterator
	{
	friend class list<T>;
	private:
		Node* iNode;
	public:
		operator++() { iNode = iNode->iNext; }
		operator--() { iNode = iNode->iPrevious; }
		T& operator*() { return iNode->iData; }
		operator==(iterator& rhs) { return iNode == rhs.iNode; }
		operator!=(iterator& rhs) { return iNode != rhs.iNode; }
	};

private:
	Node* iHead;
	Node* iTail;
	Node iEndOfSequence;
public:
	void push(const T&);
	T pop();
	bool is_empty();
	iterator begin() { iterator i; i.iNode = iHead; return i; }
	iterator end() { iterator i; i.iNode = &iEndOfSequence; return i; }

	list();
	~list();
};

template<class T>
void list<T>::push(const T& data)
{
	if (iHead == &iEndOfSequence)
	{
		iHead = new Node;
		iHead->iPrev = &iEndOfSequence;
		iHead->iNext = &iEndOfSequence;
		iHead->iData = data;
		iTail = iHead;
		iEndOfSequence.iNext = iHead;
		iEndOfSequence.iPrev = iTail;
	}
	else
	{
		iTail->iNext = new Node;
		iTail->iNext->iPrev = iTail;
		iTail = iTail->iNext;
		iTail->iNext = &iEndOfSequence;
		iTail->iData = data;
		iEndOfSequence.iNext = iHead;
		iEndOfSequence.iPrev = iTail;
	}
}

template<class T>
T list<T>::pop()
{
	if (iTail == &iEndOfSequence)
	{
		return iEndOfSequence.iData;
	}

	T data = iTail->iData;
	Node* newTail = iTail->iPrev;
	delete iTail;
	iTail = newTail;
	if (iTail != &iEndOfSequence)
		iTail->iNext = &iEndOfSequence;
	else
		iHead = &iEndOfSequence;

	iEndOfSequence.iNext = iHead;
	iEndOfSequence.iPrev = iTail;

	return data;
}

template<class T>
bool list<T>::is_empty()
{
	return iHead == &iEndOfSequence;
}

template<class T>
list<T>::list()
{
	iHead = &iEndOfSequence;
	iTail = &iEndOfSequence;
	iEndOfSequence.iNext = iHead;
	iEndOfSequence.iPrev = iTail;
}

template<class T>
list<T>::~list()
{
	while(!is_empty())
		pop();
}

#endif // _LINKED_LIST_H