
namespace std
{
template<class _II, class _Fn> inline
	_Fn for_each(_II _F, _II _L, _Fn _Op)
	{for (; _F != _L; ++_F)
		_Op(*_F);
	return (_Op); }


template<class _A, class _R>
	struct unary_function {
	typedef _A argument_type;
	typedef _R result_type;
	};
		// TEMPLATE STRUCT binary_function
template<class _A1, class _A2, class _R>
	struct binary_function {
	typedef _A1 first_argument_type;
	typedef _A2 second_argument_type;
	typedef _R result_type;
	};
	
template<class _R, class _Ty>
	class mem_fun_t : public unary_function<_Ty *, _R> {
public:
	explicit mem_fun_t(_R (_Ty::*_Pm)())
		: _Ptr(_Pm) {}
	_R operator()(_Ty *_P) const
		{return ((_P->*_Ptr)()); }
private:
	_R (_Ty::*_Ptr)();
	};

template<class _R, class _Ty, class _A>
	class mem_fun1_t : public binary_function<_Ty *, _A, _R> {
public:
	explicit mem_fun1_t(_R (_Ty::*_Pm)(_A))
		: _Ptr(_Pm) {}
	_R operator()(_Ty *_P, _A _Arg) const
		{return ((_P->*_Ptr)(_Arg)); }
private:
	_R (_Ty::*_Ptr)(_A);
	};
				// TEMPLATE FUNCTION mem_fun
	template<class _R, class InIter, class _A> inline
		mem_fun1_t<_R, InIter, _A> mem_fun(_R (InIter::*_Pm)(_A))
		{return (mem_fun1_t<_R, InIter, _A>(_Pm)); }

template<class _Bfn>
	class binder1st
	: public unary_function<_Bfn::second_argument_type,
		_Bfn::result_type> {
public:
	binder1st(const _Bfn& _X,
		const _Bfn::first_argument_type& _Y)
		: op(_X), value(_Y) {}
	_Bfn::result_type operator()(const _Bfn::second_argument_type& _X) const
		{return (op(value, _X)); }
protected:
	_Bfn op;
	_Bfn::first_argument_type value;
	};
		// TEMPLATE FUNCTION bind1st
template<class _Bfn, class _Ty> inline
	binder1st<_Bfn> bind1st(const _Bfn& _X, const _Ty& _Y)
		{return (binder1st<_Bfn>(_X,
			_Bfn::first_argument_type(_Y))); }
}			
template<typename T> 
class reference_wrapper 
{
public:
	reference_wrapper(T* _X) : ref(_X) {}

	// access
	operator T&() { return *ref; }
	
	template<typename T1> 
		typename T::result_type operator()(const T1& _X) { return ref->operator()(_X); }
	template<typename T1, typename T2> 
		typename T::result_type operator()(const T1& _X, const T2& _Y) { return ref->operator()(_X, _Y); }
	template<typename T3>
		operator T3() { return ref->operator T3(); }

protected:
	T* ref;
};

template<typename T> reference_wrapper<T> ref(T* _X) 
	{ return reference_wrapper<T>(_X); }
template<typename T> reference_wrapper<const T> cref(const T* _X)
	{ return reference_wrapper<const T>(_X); }

template<class Oper, class InIter>
	class input2nd_t
	: public std::unary_function<typename Oper::first_argument_type,
		typename Oper::result_type> {
public:
	input2nd_t(const Oper& _X, InIter _Y)
		: op(_X), iter(_Y), n(0) {}
	~input2nd_t() { n = 0; }
	Oper::result_type operator()(const Oper::first_argument_type& _X)
		{n++; return (op(_X, *iter++)); }
	operator int() const { return n; }
protected:
	Oper op;
	InIter iter;
	int n;
	};
template<class Oper, class InIter> inline
	input2nd_t<Oper, InIter> input2nd(const Oper& _X, InIter _Y)
		{return (input2nd_t<Oper, InIter>(_X, _Y)); }
