// GameCard.cpp : implementation file
//

#include "stdafx.h"
#include "2Patient.h"
#include "GameCardRenderer.h"

#include "Engine/GameCard.h"
#include "Engine/GameCardStack.h"
#include "Engine/GameBoard.h"
#include "Engine/GameModel.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define _countof(Images) (sizeof(Images)/sizeof(Images[0]))

/////////////////////////////////////////////////////////////////////////////
// CGameCardRenderer

CGameCardRenderer::CGameCardRenderer(CGameModel& aModel) : 
	iGameModel(aModel)
{
}

CGameCardRenderer::~CGameCardRenderer()
{
	delete iCardDC;
	DeleteObject(iCardImagesBitmap);
}

bool CGameCardRenderer::InitResources()
{
/*	static const char szCardsDLLName[] = "\\cards.dll";
	char szCardsDLLPath[_MAX_PATH+14]; // Note: 8.3 name
	GetSystemDirectoryA(szCardsDLLPath, _countof(szCardsDLLPath));
	lstrcatA(szCardsDLLPath, szCardsDLLName);
	HINSTANCE hCardsDLL = LoadLibraryA(szCardsDLLPath);

	if (hCardsDLL == NULL)
		return false;

	for (int i = 0; i < 52; ++i)
	{
		iCardBitmaps[i] = ::LoadBitmap(hCardsDLL, MAKEINTRESOURCE(i+1));
	}

	iCardFaceDownBitmap = ::LoadBitmap(hCardsDLL, MAKEINTRESOURCE(53));

	iCardBitmapMask = ::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_CARDMASK));

	FreeLibrary(hCardsDLL);
*/
	iCardImagesBitmap = ::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_CARDS));
	BITMAP bitmap;
	CBitmap::FromHandle(iCardImagesBitmap)->GetBitmap(&bitmap);

	iCardBitmapSize.cx = bitmap.bmWidth / 13;
	iCardBitmapSize.cy = bitmap.bmHeight / 5;

	static_cast<CGameBoard&>(iGameModel.GameBoard()).CardImageSize().SetSpacialInformation(0, 0, iCardBitmapSize.cx, iCardBitmapSize.cy);

	iCardDC = new CDC;
	iCardDC->CreateCompatibleDC(NULL);
	
	return true;
}

void CGameCardRenderer::DrawCardStack(CDC& aTargetDC, const CRect& aTargetRect, const CGameCardStack& aGameCardStack)
{
	if (aGameCardStack.CardCount() > 0)
	{
		int i = 0;
		if (aGameCardStack.Type() != CGameCardStack::EInStack)
		{
			i = aGameCardStack.CardCount() - 2; // draw top card and card underneath top card only
			if (i < 0)
				i = 0;
		}	

		for (; i < aGameCardStack.CardCount(); ++i)
		{
			if (aGameCardStack.Card(i).RectIntersects(aTargetRect.left, aTargetRect.top, 
				aTargetRect.right, aTargetRect.bottom))
				DrawCard(aTargetDC, aGameCardStack.Card(i));
		}
		return;
	}

	int ix(0), iy(iCardBitmapSize.cy * 4);
	switch(aGameCardStack.Type())
	{
	case CGameCardStack::EOutStack:
		ix = iCardBitmapSize.cx * 10;
		break;
	case CGameCardStack::EInStack:
		ix = iCardBitmapSize.cx * 12;
		break;
	case CGameCardStack::EPlayer1InHand:
	case CGameCardStack::EPlayer2InHand:
		ix = iCardBitmapSize.cx * 11;
		break;
	case CGameCardStack::EPlayer1OutHand:
	case CGameCardStack::EPlayer2OutHand:
		ix = iCardBitmapSize.cx * 11;
		break;
	default:
		return;
	}

	int x,y,cx,cy;
	static_cast<const ISpacialInformation&>(aGameCardStack).SpacialInformation(x, y, cx, cy);

	CBitmap* oldBitmap = iCardDC->SelectObject(CBitmap::FromHandle(iCardImagesBitmap));

	aTargetDC.StretchBlt(x + (cx - iCardBitmapSize.cx) / 2, y + (cy - iCardBitmapSize.cy) / 2, 
		iCardBitmapSize.cx, iCardBitmapSize.cy, iCardDC, 
		0, iCardBitmapSize.cy * 4, iCardBitmapSize.cx, iCardBitmapSize.cy, SRCAND);
	aTargetDC.StretchBlt(x + (cx - iCardBitmapSize.cx) / 2, y + (cy - iCardBitmapSize.cy) / 2, 
		iCardBitmapSize.cx, iCardBitmapSize.cy, iCardDC, ix, iy, 
		iCardBitmapSize.cx, iCardBitmapSize.cy, SRCPAINT);
	iCardDC->SelectObject(oldBitmap);
}

void CGameCardRenderer::DrawCard(CDC& aTargetDC, const CGameCard& aGameCard)
{
	int x,y,cx,cy;
	static_cast<const ISpacialInformation&>(aGameCard).SpacialInformation(x, y, cx, cy);

	CBitmap* oldBitmap = iCardDC->SelectObject(CBitmap::FromHandle(iCardImagesBitmap));

	aTargetDC.StretchBlt(x + (cx - iCardBitmapSize.cx) / 2, y + (cy - iCardBitmapSize.cy) / 2, 
		iCardBitmapSize.cx, iCardBitmapSize.cy, iCardDC, 
		0, iCardBitmapSize.cy * 4, iCardBitmapSize.cx, iCardBitmapSize.cy, SRCAND);
	aTargetDC.StretchBlt(x + (cx - iCardBitmapSize.cx) / 2, y + (cy - iCardBitmapSize.cy) / 2, 
		iCardBitmapSize.cx, iCardBitmapSize.cy, iCardDC, 
		aGameCard.FaceDown() ? iCardBitmapSize.cx : aGameCard.Value() * iCardBitmapSize.cx,
		aGameCard.FaceDown() ? iCardBitmapSize.cy * 4 : aGameCard.Suit() * iCardBitmapSize.cy,
		iCardBitmapSize.cx, iCardBitmapSize.cy, SRCPAINT);
	iCardDC->SelectObject(oldBitmap);
}