// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "2Patient.h"
#include "2PatientDoc.h"

#include "MainFrm.h"
#include "OptionsDialog.h"

#include "Engine/GameModel.h"
#include "Engine/GamePlayer.h"

#include "Engine/logging.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame()
{	
}

CMainFrame::~CMainFrame()
{
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	CCreateContext* pContext = (CCreateContext*)lpCreateStruct->lpCreateParams;

	C2PatientDoc* pDoc = static_cast<C2PatientDoc*>(pContext->m_pCurrentDoc);
	ASSERT_VALID(pDoc);

	pDoc->Model().AddObserver(*this);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	cs.cx = 220;
	cs.cy = 396;

	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnGameStateChanged(CGameModel& aModel, EGameState aOldState, EGameState aNewState)
{
	switch(aNewState)
	{
	case EGameStateUnknown:
		m_nCurrentStatus = 0;
		break;
	case EGameStateDealing:
		m_nCurrentStatus = 1;
		break;
	case EGameStatePlayer1ToMove:
		m_nCurrentStatus = (aOldState != EGameStateThinking ? 2 : 5);
		break;
	case EGameStatePlayer1MakingMove:
		m_nCurrentStatus = 3;
		break;
	case EGameStatePlayer1MadeMove:
		m_nCurrentStatus = 4;
		break;
	case EGameStatePlayer1Thinking:
		m_nCurrentStatus = 5;
		break;
	case EGameStatePlayer2ToMove:
		m_nCurrentStatus = (aOldState != EGameStateThinking ? 6 : 9);
		break;
	case EGameStatePlayer2MakingMove:
		m_nCurrentStatus = 7;
		break;
	case EGameStatePlayer2MadeMove:
		m_nCurrentStatus = 8;
		break;
	case EGameStatePlayer2Thinking:
		m_nCurrentStatus = 9;
		break;
	case EGameStateGameOver:
		m_nCurrentStatus = 10;
		break;
	case EGameStatePlayer1Won:
		m_nCurrentStatus = 11;
		break;
	case EGameStatePlayer2Won:
		m_nCurrentStatus = 12;
		break;
	}

	SetMessageText(m_strStatusTexts[m_nCurrentStatus]);
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext) 
{
	for (int i = 0; i < 13; ++i)
	{
		m_strStatusTexts[i].LoadString(IDS_STATUS_UNKNOWN + i);
	}

	return CFrameWnd::OnCreateClient(lpcs, pContext);
}

void CMainFrame::GetMessageString(UINT nID, CString& rMessage) const
{
	if (nID == AFX_IDS_IDLEMESSAGE)
	{
		rMessage = m_strStatusTexts[m_nCurrentStatus];
	}
	else
	{
		CFrameWnd::GetMessageString(nID, rMessage);
	}
}
