// Simple.h
// ------------
//
// Copyright (c) 2002 Symbian Ltd.  All rights reserved.
//

////////////////////////////////////////////////////////////////////
// Simple
// ----------
//
//
// The class definitions for the simple example application
// containing a single view.
//
// The class definitions are:
//
// CSimpleApplication 
// CSimpleAppUi
// CSimpleAppView
// CSimpleDocument
//
//
////////////////////////////////////////////////////////////////////
#ifndef __Simple_H
#define __Simple_H

#include <coeccntx.h>

#include <eikenv.h>
#include <qikdocument.h>
#include <qikapplication.h>
#include <qikappui.h>
#include <eikmenup.h>
#include <eikdialg.h>
#include <eikchlst.h>
#include <qikslider.h>
#include <eikchkbx.h>

#include <eikon.hrh>

#include <2Patient.rsg>
#include "Simple.hrh"

class CSimpleDocument;

#include "GameState.h"
#include "GameBoardRenderer.h"

class CGameModel;
class CGameCardRenderer;


////////////////////////////////////////////////////////////////////////
//
// CSimpleApplication
//
////////////////////////////////////////////////////////////////////////

class CSimpleApplication : public CQikApplication
	{
private: 
	           // Inherited from class CApaApplication
	CApaDocument* CreateDocumentL();
	TUid AppDllUid() const;
	};

////////////////////////////////////////////////////////////////////////
//
// CSimpleAppView
//
////////////////////////////////////////////////////////////////////////
class CSimpleAppView : public CCoeControl, private IGameBoardRenderer, private IGameStateObserver
    {
public:
	static CSimpleAppView* NewL(CSimpleDocument& aDoc, const TRect& aRect);
	CSimpleAppView(CSimpleDocument& aDoc);
	~CSimpleAppView();
    void ConstructL(const TRect& aRect);

public:
	void UpdateL(CSimpleDocument& aDoc);

private:
	           // Inherited from CCoeControl
	virtual void Draw(const TRect& /*aRect*/) const;
	virtual void HandlePointerEventL(const TPointerEvent& aPointerEvent); 
	virtual void SizeChanged();
		// Inherited from IGameBoardRenderer
	virtual void RenderGameBoard(const SSpacialInformation& aRect);	
		// Inherited from IGameStateObserver
	virtual void OnSetGameState(CGameModel& aModel, EGameState aNewState);
	virtual void OnGameStateChanged(CGameModel& aModel, EGameState aOldState, EGameState aNewState);

	void OnMouseMove( const TPoint& aPoint);
	void OnLButtonDown(const TPoint& aPoint);
	void OnLButtonUp(const TPoint& aPoint);
	void OnLButtonDblClk(const TPoint& aPoint); 

	void UpdateSpacialInformation();
	
	class CGameStateTimerOneShot : public CTimer
	{
	public:
		static CGameStateTimerOneShot* NewLC(CSimpleAppView& aParent);
		static CGameStateTimerOneShot* NewL(CSimpleAppView& aParent);
		void FireL();
		void RunL();
	private:
		CGameStateTimerOneShot(CSimpleAppView& aParent);
		void ConstructL();
		
	private:
		CSimpleAppView& iParent;
	} *iGameStateTimerOneShot;

	friend class CGameStateTimerOneShot;

private:
	TBool iConstructing;
	CDesC16ArrayFlat* iMessageText;
	TBool iRedrawWindow;
	TRect iRedrawRect;
	CSimpleDocument& iDoc;
	CFbsBitGc* iCanvasGc;
	CFbsBitmap* iCanvasBitmap;
	CFbsBitmapDevice* iCanvasBitmapDevice;
	
	bool iDrawGameState;
    };


////////////////////////////////////////////////////////////////////////
//
// CSimpleAppUi
//
////////////////////////////////////////////////////////////////////////
class CSimpleAppUi : public CQikAppUi
    {
public:
   CSimpleAppUi(CSimpleDocument& aDoc) : CQikAppUi(), iDoc(aDoc) {}
    void ConstructL();
	~CSimpleAppUi();

public:
	void UpdateAllViewsL(CSimpleDocument& aDoc);

private:
     // Inherited from class CEikAppUi
	void HandleCommandL(TInt aCommand);

#if defined(_DEBUG)
	/* Only used in this application to add a Close item in Debug mode */
	void DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane);
#endif


private:
	CSimpleDocument& iDoc;
	CSimpleAppView* iAppView;
	};
	
class CCoeEnv;


////////////////////////////////////////////////////////////////////////
//
// CSimpleDocument
//
////////////////////////////////////////////////////////////////////////
class CSimpleDocument : public CQikDocument, private IGameStateObserver
	{
public:
	CSimpleDocument(CEikApplication& aApp, CCoeEnv* aCoeEnv);
	~CSimpleDocument();

public:
	CGameModel& Model() { return *iGameModel; }
	CGameCardRenderer& GameCardRenderer() { return *iGameCardRenderer; }

private:
friend class CSimpleAppUi;
	void AppUiConstructed();
	void OnNewDocument();

private: 
    // Inherited from CEikDocument
	CEikAppUi* CreateAppUiL();

protected:
// from IGameStateObserver
	virtual void OnSetGameState(CGameModel& aModel, EGameState aNewState);
	virtual void OnGameStateChanged(CGameModel& aModel, EGameState aOldState, EGameState aNewState);

private:
	CCoeEnv* iCoeEnv;
	CSimpleAppUi* iAppUi;
	CGameModel* iGameModel;
	CGameCardRenderer* iGameCardRenderer;
	TBool iModified;
	
	class CGameStateDistributor : public CActive
		{
		public:
		CGameStateDistributor(CSimpleDocument& aDocument) : CActive(EPriorityStandard), iParent(aDocument) {}
		virtual void Pump();
		protected:	
		virtual void DoCancel();
		virtual void RunL();
		CSimpleDocument& iParent;
		} *iDistributor;
		
	friend class CGameStateDistributor;
	};


class COptionsDialog : public CEikDialog
	{
public:
	COptionsDialog(CSimpleDocument& aDocument);

private: 
	// from CEikDialog
	void PreLayoutDynInitL();
	void HandleControlStateChangeL(TInt aControlId);
	TBool OkToExitL(TInt aButtonId);
	// Checks settings are valid
	void ValidateControlState();

private:
	CSimpleDocument& iDocument;
	CEikChoiceList* iNumberOfPlayersList;
	CEikChoiceList* iNumberOfCardStacksList;
	CQikSlider* iCardAnimationSpeed;
	CEikCheckBox* iAllowISOOS;
	};

#endif

