// CSimpleAppUi.cpp
// -----------------
//
// Copyright (c) 2002 Symbian Ltd.  All rights reserved.
//

////////////////////////////////////////////////////////////////////////
//
// Source file for the implementation of the 
// application UI class - CSimpleAppUi
//
////////////////////////////////////////////////////////////////////////

#include "Simple.h"
#include "GameModel.h"

//             The second phase constructor of the application UI class.
//             The application UI creates and owns the one and only view.
// 
void CSimpleAppUi::ConstructL()
    {
	           // BaseConstructL() completes the UI framework's
	           // construction of the App UI.
    BaseConstructL();
	           // Create the single application view in which to
	           // draw the text "Figure 1!", passing into it
	           // the rectangle available to it.
	
	TRect rect = ClientRect();           
	           
	iAppView = CSimpleAppView::NewL(iDoc, rect);
	
	iDoc.AppUiConstructed();
	}
	
void CSimpleAppUi::UpdateAllViewsL(CSimpleDocument& aDoc)
{
	iAppView->UpdateL(aDoc);
}


//             The app Ui owns one view and is. 
//             therefore, responsible for destroying it
//
CSimpleAppUi::~CSimpleAppUi()
	{
	delete iAppView;
	}


//             Called by the UI framework when a command has been issued.
//             In this example, a command can originate  by selection of a menu item.
//             The command Ids are defined in the .hrh file
//             and are 'connected' to the hot-key and menu item in the
//             resource file.
//             Note that the EEikCmdExit is defined by the UI
//             framework and is pulled in by including eikon.hrh
//
void CSimpleAppUi::HandleCommandL(TInt aCommand)
	{
	switch (aCommand)
		{
	case ECmdNewGame:
		static_cast<IGameObject&>(iDoc.Model()).NewGame(iDoc.Model().NumberOfPlayersNextGame());
		break;
	case ECmdOptions:
		{
		CEikDialog* dialog = new(ELeave) COptionsDialog(iDoc);
		dialog->ExecuteLD(R_OPTIONS_DIALOG);
		} break;
	case EEikCmdExit: 
		Exit();
		break;
		}
	}

#if defined(_DEBUG)
void CSimpleAppUi::DynInitMenuPaneL(TInt aMenuId, CEikMenuPane* aMenuPane)
	{
	if(aMenuId == R_SIMPLE_FIRST_MENU)
		{
		CEikMenuPaneItem::SData itemData;
		TBuf<16> closeCommandText;
		iEikonEnv->ReadResource(closeCommandText, R_SIMPLE_MENU_CLOSE);
		itemData.iText=closeCommandText;
		itemData.iCommandId=EEikCmdExit;
		itemData.iFlags=0;
		itemData.iCascadeId=0;
		aMenuPane->AddMenuItemL(itemData);
		}
	}
#endif

COptionsDialog::COptionsDialog(CSimpleDocument& aDocument):
	iDocument(aDocument)
	{
	}

// Initialise dialog with current settings
void COptionsDialog::PreLayoutDynInitL()
	{
	iNumberOfPlayersList = static_cast<CEikChoiceList*>(Control(EIdNumberOfPlayers));
	iNumberOfCardStacksList = static_cast<CEikChoiceList*>(Control(EIdNumberOfCardStacks));
	iCardAnimationSpeed = static_cast<CQikSlider*>(Control(EIdCardAnimationSpeed));
	iAllowISOOS = static_cast<CEikCheckBox*>(Control(EIdAllowISOOS));

	iNumberOfCardStacksList -> SetCurrentItem(iDocument.Model().NumberOfInStacksNextGame() == 4 ? 0 : 1);
	iNumberOfPlayersList -> SetCurrentItem(iDocument.Model().NumberOfPlayersNextGame());
	iCardAnimationSpeed -> SetValue(iDocument.Model().CardSpeed());
	iAllowISOOS -> SetState(iDocument.Model().AllowISOOSNextGame() ? 
		CEikButtonBase::ESet : CEikButtonBase::EClear);

	ValidateControlState();
	}

void COptionsDialog::HandleControlStateChangeL(TInt /*aControlId*/)
	{
	ValidateControlState();
	}

// Check settings are valid
void COptionsDialog::ValidateControlState()
	{
	}

// Save dialog settings
TBool COptionsDialog::OkToExitL(TInt /*aButtonId*/)
	{
	iDocument.Model().SetNumberOfInStacksNextGame(
		iNumberOfCardStacksList -> CurrentItem() == 0 ? 4 : 7);
	iDocument.Model().SetNumberOfPlayersNextGame(
		iNumberOfPlayersList -> CurrentItem());
	iDocument.Model().SetCardSpeed(
		iCardAnimationSpeed->CurrentValue());
	iDocument.Model().SetAllowISOOSNextGame(
		iAllowISOOS->State() == CEikButtonBase::ESet ? true : false);
	return ETrue;
	}


