// CSimpleAppView.cpp
// -------------------
//

// Copyright (c) 2002 Symbian Ltd.  All rights reserved.
// 

////////////////////////////////////////////////////////////////////////
//
// Source file for the implementation of the 
// application view class - CSimpleAppView
//
////////////////////////////////////////////////////////////////////////

#include "Simple.h"

#include "GameModel.h"
#include "GameCardStack.h"
#include "GameBoard.h"
#include "GamePlayer.h"
#include "GameCardRenderer.h"

enum { MAX_WIDTH = 240, MAX_HEIGHT = 320 };

//
//             Constructor for the view. 
// 
CSimpleAppView::CSimpleAppView(CSimpleDocument& aDoc) : iDoc(aDoc)
	{
	TPckgBuf<int> intBuf;
	}
	
CSimpleAppView::~CSimpleAppView()
{
	delete iMessageText;
	delete iCanvasBitmap;
	delete iCanvasBitmapDevice;
	delete iCanvasGc;
}


//             Static NewL() function to start the standard two
//             phase construction.
//
CSimpleAppView* CSimpleAppView::NewL(CSimpleDocument& aDoc, const TRect& aRect)
	{
	CSimpleAppView* self = new(ELeave) CSimpleAppView(aDoc);
	CleanupStack::PushL(self);
	self->ConstructL(aRect);
	CleanupStack::Pop();
	return self;
	}


//             Second phase construction.
//
void CSimpleAppView::ConstructL(const TRect& aRect)
    {	
    iConstructing = ETrue;
    	
	iMessageText = iEikonEnv->ReadDesC16ArrayResourceL(R_MESSAGE_TEXT);	

    iGameStateTimerOneShot = CGameStateTimerOneShot::NewL(*this);
    
    TSize size(MAX_WIDTH, MAX_HEIGHT);
	iCanvasBitmap=new(ELeave) CFbsBitmap(); //Create a bitmap to store the window content
	User::LeaveIfError(iCanvasBitmap->Create(size,EColor64K)); //ditto
	iCanvasBitmapDevice=CFbsBitmapDevice::NewL(iCanvasBitmap); //Create a bitmap device so we can draw to the bitmap
	iCanvasGc=CFbsBitGc::NewL(); //Create a GC to draw to the bitmap
	iCanvasGc->Activate(iCanvasBitmapDevice); //Active it on the bitmap we just created
	
//CFbsBitGc* gc;
//User::LeaveIfError(CONST_CAST(CFbsScreenDevice*,iFunctions->ScreenDevice())->CreateContext(gc)); //Create a screen gc
//iBitmapGc->BitBlt(TPoint(0,0),*gc,TRect(iWindowTopLeft,size)); //Copy the window content 

	CGameModel& model = iDoc.Model();
	model.GameBoard().SetGameBoardRenderer(this);

	           // Control is a window owning control
	CreateWindowL();
	           // Extent of the control. This is
	           // the whole rectangle available to application.
	           // The rectangle is passed to us from the application UI.
	SetRect(aRect);
			   // At this stage, the control is ready to draw so
	           // we tell the UI framework by activating it.
	ActivateL();
	
	model.AddObserver(*this);
	
	iConstructing = EFalse;
	}


//             Drawing the view - 
//
void CSimpleAppView::Draw(const TRect& aRect) const
	{
               // Window graphics context
	CWindowGc& gc = SystemGc();

	CGameModel& model = iDoc.Model();
	CGameCardRenderer& renderer = iDoc.GameCardRenderer();
		
	TRect  rectClient;

	if (!iRedrawWindow)
		rectClient = aRect;
	else
		rectClient = iRedrawRect;

	TRect rect(rectClient);
	if (rect.iBr.iX >= MAX_WIDTH)
		rect.iBr.iX = MAX_WIDTH - 1;
	if (rect.iBr.iY >= MAX_HEIGHT)
		rect.iBr.iY  = MAX_HEIGHT - 1;

	iCanvasGc->SetBrushColor(TRgb(0, 128, 0));
	iCanvasGc->SetBrushStyle(CGraphicsContext::ESolidBrush);
	iCanvasGc->SetPenStyle(CGraphicsContext::ENullPen);
	iCanvasGc->DrawRect(rect);

	// rendering optimzation (todo), create render stack method which
	// ensures that cards under other cards are not drawn

	CGameCardStack *stackToDrawLast = 0;
	if (model.GameState() == EGameStatePlayer1MakingMove ||
		model.GameState() == EGameStatePlayer2MakingMove)
	{
		stackToDrawLast = &model.GameBoard().CardStackMove();
	}

	for (int i = 0; i < model.StackCount(); ++i)
	{
		CGameCardStack& stack = static_cast<CGameCardStack&>(model.CardStack(i));
		if (&stack == stackToDrawLast)
			continue;
		renderer.DrawCardStack(*iCanvasGc, rect, stack);
	}
	
	if (iDrawGameState)
	{
		TInt textIndex = -1;
		
		switch(model.GameState())
		{
		case EGameStateUnknown:
			textIndex = -1;//0;
			break;
		case EGameStateDealing:
			textIndex = -1;//1;
			break;
		case EGameStatePlayer1ToMove:
			textIndex = model.GameStatePrevious() & EGameStatePlayer1 || model.GamePlayerOne().ComputerPlayer() ? -1 : ((model.GameStatePrevious() != EGameStateThinking ? 2 : 5));
			break;
		case EGameStatePlayer1MakingMove:
			textIndex = -1;//3;
			break;
		case EGameStatePlayer1MadeMove:
			textIndex = -1;//4;
			break;
		case EGameStatePlayer1Thinking:
			textIndex = -1;//5;
			break;
		case EGameStatePlayer2ToMove:
			textIndex = model.GameStatePrevious() & EGameStatePlayer2 || model.GamePlayerTwo().ComputerPlayer() ? -1 : ((model.GameStatePrevious() != EGameStateThinking ? 6 : 9));
			break;
		case EGameStatePlayer2MakingMove:
			textIndex = -1;//7;
			break;
		case EGameStatePlayer2MadeMove:
			textIndex = -1;//8;
			break;
		case EGameStatePlayer2Thinking:
			textIndex = -1;//9;
			break;
		case EGameStateGameOver:
			textIndex = 10;
			break;
		case EGameStatePlayer1Won:
			textIndex = 11;
			break;
		case EGameStatePlayer2Won:
			textIndex = 12;
			break;
		default:
			textIndex = 0;
			break;
		}
		
		if (textIndex != -1)
		{
			iCanvasGc->UseFont(iEikonEnv->TitleFont()); // use the system message font
			iCanvasGc->SetPenStyle(CGraphicsContext::ESolidPen);
			TRgb black(0,0,0);
			iCanvasGc->SetPenColor(black);	
			iCanvasGc->DrawText((*iMessageText)[textIndex], TPoint(19, 209));
			iCanvasGc->DrawText((*iMessageText)[textIndex], TPoint(21, 211));
			iCanvasGc->DrawText((*iMessageText)[textIndex], TPoint(19, 211));
			iCanvasGc->DrawText((*iMessageText)[textIndex], TPoint(21, 209));
			TRgb white(255,255,255);
			iCanvasGc->SetPenColor(white);
			iCanvasGc->DrawText((*iMessageText)[textIndex], TPoint(20, 210));
		}
	}

	if (stackToDrawLast)
	{
		renderer.DrawCardStack(*iCanvasGc, rect, *stackToDrawLast);
	}

	gc.BitBlt(rectClient.iTl, iCanvasBitmap, rectClient);

	ControlEnv()->WsSession().Flush();
	}

void CSimpleAppView::RenderGameBoard(const SSpacialInformation& aRect)
{
	iRedrawWindow = true;
	iRedrawRect.iTl.iX = aRect.iXPosition;
	iRedrawRect.iTl.iY = aRect.iYPosition;
	iRedrawRect.iBr.iX = iRedrawRect.iTl.iX + aRect.iWidth - 1;
	iRedrawRect.iBr.iY = iRedrawRect.iTl.iY + aRect.iHeight - 1;
	DrawNow();
	ControlEnv()->WsSession().Flush();
	iRedrawWindow = false;
}

void CSimpleAppView::OnSetGameState(CGameModel& aModel, EGameState aNewState)
{
}

void CSimpleAppView::OnGameStateChanged(CGameModel& aModel, EGameState aOldState, EGameState aNewState)
{
	iGameStateTimerOneShot->FireL();
}

CSimpleAppView::CGameStateTimerOneShot::CGameStateTimerOneShot(CSimpleAppView& aParent)
	: CTimer(CActive::EPriorityStandard), iParent(aParent)
{
}

CSimpleAppView::CGameStateTimerOneShot* CSimpleAppView::CGameStateTimerOneShot::NewLC(CSimpleAppView& aParent)
	{
	CGameStateTimerOneShot* self=new (ELeave) CGameStateTimerOneShot(aParent);
	CleanupStack::PushL(self);
	self->ConstructL();
	return self;
	}

CSimpleAppView::CGameStateTimerOneShot* CSimpleAppView::CGameStateTimerOneShot::NewL(CSimpleAppView& aParent)
	{
	CGameStateTimerOneShot* self = NewLC(aParent);
	CleanupStack::Pop();
	return self;
	}

void CSimpleAppView::CGameStateTimerOneShot::ConstructL()
{
	CTimer::ConstructL();
	CActiveScheduler::Add(this); 
}

void CSimpleAppView::CGameStateTimerOneShot::FireL()
{
	if (iParent.iConstructing)
		return;

	iParent.iDrawGameState = true;
			
	if (IsActive())
		Cancel();
		
	CTimer::After(2*1000*1000);

	iParent.UpdateSpacialInformation();
	iParent.iRedrawWindow = true;
	iParent.iRedrawRect = iParent.Rect();
	iParent.DrawNow();
	iParent.ControlEnv()->WsSession().Flush();
	iParent.iRedrawWindow = false;
}

void CSimpleAppView::CGameStateTimerOneShot::RunL()
{
//	iParent.UpdateSpacialInformation();

	iParent.iDrawGameState = false;

	iParent.iRedrawWindow = true;
	iParent.iRedrawRect = iParent.Rect();
	iParent.DrawNow();
	iParent.ControlEnv()->WsSession().Flush();
	iParent.iRedrawWindow = false;
}

void CSimpleAppView::UpdateL(CSimpleDocument& aDoc)
{
//	if (!lHint)
		UpdateSpacialInformation();

	iRedrawWindow = true;
	iRedrawRect = Rect();
	DrawNow();
	ControlEnv()->WsSession().Flush();
	iRedrawWindow = false;
}

void CSimpleAppView::UpdateSpacialInformation()
{
	TRect rect =Rect();

	if (rect.iBr.iX > MAX_WIDTH - 1)
		rect.iBr.iX = MAX_WIDTH - 1;
	if (rect.iBr.iY > MAX_HEIGHT - 1)
		rect.iBr.iY = MAX_HEIGHT - 1;
		
	ISpacialInformation& gameSpace = static_cast<ISpacialInformation&>(iDoc.Model().GameBoard());
	gameSpace.SetSpacialInformation(0, 0, rect.iBr.iX, rect.iBr.iY);
}

void CSimpleAppView::SizeChanged() 
{
	CCoeControl::SizeChanged();

	UpdateSpacialInformation();

	DrawDeferred();	
}

void CSimpleAppView::HandlePointerEventL(const TPointerEvent& aPointerEvent) 
{
	switch(aPointerEvent.iType)
	{
	case TPointerEvent::EButton1Down:
		OnLButtonDown(aPointerEvent.iPosition);
		Window().PointerFilter (EPointerFilterDrag,	0);
		break;
	case TPointerEvent::EButton1Up:
		OnLButtonUp(aPointerEvent.iPosition);
		Window().PointerFilter (EPointerFilterDrag,	EPointerFilterDrag);
		break;
	case TPointerEvent::EDrag:
		OnMouseMove(aPointerEvent.iPosition);
		break;
	default:
		break;
	}
}

void CSimpleAppView::OnMouseMove( const TPoint& aPoint)
{
	if (iDoc.Model().CurrentPlayer().ComputerPlayer())
		return;

	if (iDoc.Model().GameState() != EGameStatePlayer1MakingMove &&
		iDoc.Model().GameState() != EGameStatePlayer2MakingMove)
	{
		return;
	}

	int x1, y1, x2, y2;
	iDoc.Model().GameBoard().SelectionBoundingRect(x1, y1, x2, y2, true);
	iDoc.Model().GameBoard().MoveSelection(aPoint.iX, aPoint.iY);
	iDoc.Model().GameBoard().SelectionBoundingRect(x1, y1, x2, y2, false);

	TRect rect(TPoint(x1, y1), TPoint(x2+1, y2+1));
	iRedrawWindow = true;
	iRedrawRect = rect;
	DrawNow();
	iRedrawWindow = false;
}

void CSimpleAppView::OnLButtonDown(const TPoint& aPoint) 
{
	if (iDoc.Model().CurrentPlayer().ComputerPlayer())
		return;

	if (iDoc.Model().GameBoard().StartMove(aPoint.iX,aPoint.iY))
	{
//		SetCursor(iMoveCardCursor);
//		SetCapture();
	}
}

void CSimpleAppView::OnLButtonUp(const TPoint& aPoint) 
{
	if (iDoc.Model().CurrentPlayer().ComputerPlayer())
		return;

//	ReleaseCapture();

	if (iDoc.Model().GameState() == EGameStatePlayer1MakingMove ||
		iDoc.Model().GameState() == EGameStatePlayer2MakingMove)
	{
		iDoc.Model().GameBoard().EndMove(aPoint.iX,aPoint.iY);
	}

	UpdateSpacialInformation();
	
	DrawDeferred();
}

void CSimpleAppView::OnLButtonDblClk(const TPoint& aPoint) 
{
	if (iDoc.Model().CurrentPlayer().ComputerPlayer())
		return;

	if (iDoc.Model().GameBoard().StartMove(aPoint.iX, aPoint.iY, true))
	{
		iDoc.Model().GameBoard().EndMove(aPoint.iX, aPoint.iY, true);
	}
}

