// CSimpleDocument.cpp
// ---------------------
//
// Copyright (c) 2002 Symbian Ltd.  All rights reserved.
//

////////////////////////////////////////////////////////////////////////
//
// Source file for the implementation of the 
// document class - CSimpleDocument
//
////////////////////////////////////////////////////////////////////////

#include "Simple.h"

#include "GameModel.h"
#include "Base64.h"
#include "GameCardRenderer.h"

#include <eikenv.h>


//             The constructor of the document class just passes the
//             supplied reference to the constructor initialisation list.
//             The document has no real work to do in this application.
//
CSimpleDocument::CSimpleDocument(CEikApplication& aApp, CCoeEnv* aCoeEnv)
		: CQikDocument(aApp), iCoeEnv(aCoeEnv)
	{
	}
	
CSimpleDocument::~CSimpleDocument()
{
	delete iGameModel;
	delete iGameCardRenderer;
}


//             This is called by the UI framework as soon as the 
//             document has been created. It creates an instance
//             of the ApplicationUI. The Application UI class is
//             an instance of a CEikAppUi derived class.
//
CEikAppUi* CSimpleDocument::CreateAppUiL()
	{
		iGameModel = new (ELeave) CGameModel();
		static_cast<IGameObject*>(iGameModel)->ConstructL();

    		 iAppUi = new(ELeave) CSimpleAppUi(*this);
		
		return iAppUi;
	}
	
void CSimpleDocument::AppUiConstructed()
{
	iGameCardRenderer = new (ELeave) CGameCardRenderer(*iGameModel);
	iGameCardRenderer->ConstructL();

	iGameModel->AddObserver(*this);
	
	iDistributor = new (ELeave)  CGameStateDistributor(*this);
	CActiveScheduler::Add(iDistributor);
	
	OnNewDocument();
}

void CSimpleDocument::OnNewDocument()
{
	static_cast<IGameObject*>(iGameModel)->NewGame(iGameModel->NumberOfPlayersNextGame());

	iModified = EFalse;
}

void CSimpleDocument::OnSetGameState(CGameModel& aModel, EGameState aNewState)
{
	if (iDistributor->IsActive())
		return;
	
	iDistributor->Pump();	
}

void CSimpleDocument::OnGameStateChanged(CGameModel& aModel, EGameState aOldState, EGameState aNewState)
{
	TRAPD(err, iAppUi->UpdateAllViewsL(*this));
}

void CSimpleDocument::CGameStateDistributor::Pump()
{
	SetActive();
	TRequestStatus* status = &iStatus;
	User::RequestComplete(status, KErrNone);
}	

void CSimpleDocument::CGameStateDistributor::DoCancel()
{
}

void CSimpleDocument::CGameStateDistributor::RunL()
{
	CGameState& state = static_cast<CGameState&>(iParent.Model());

//	if (!iModalDialog)
		if (state.NeedNotification())
			state.NotifyObserversStateChanged();
}

