// GameCardRenderer.cpp : implementation file
//

#include <eikenv.h>
#include "GameCardRenderer.h"

#include "GameCard.h"
#include "GameCardStack.h"
#include "GameBoard.h"
#include "GameModel.h"

#define _countof(Images) (sizeof(Images)/sizeof(Images[0]))

/////////////////////////////////////////////////////////////////////////////
// CGameCardRenderer

CGameCardRenderer::CGameCardRenderer(CGameModel& aModel) : 
	iGameModel(aModel)
{
}

CGameCardRenderer::~CGameCardRenderer()
{
	delete iCardImagesBitmap;
	delete iCardImagesBitmapMask;
}

bool CGameCardRenderer::ConstructL()
{
#ifdef __WINS__
	_LIT(KMbmFile, "Z:\\system\\apps\\2patient\\2patient.mbm");
#else
	//todo - might not be C:
	_LIT(KMbmFile, "C:\\system\\apps\\2patient\\2patient.mbm");
#endif
	iCardImagesBitmap = new (ELeave) CFbsBitmap;
	iCardImagesBitmapMask = new (ELeave) CFbsBitmap;
	iCardImagesBitmap->Load(KMbmFile, 0);
	iCardImagesBitmapMask->Load(KMbmFile, 1);

	iCardBitmapSize = iCardImagesBitmap->SizeInPixels();
	iCardBitmapSize.iWidth /= 13;
	iCardBitmapSize.iHeight /= 5;

	static_cast<CGameBoard&>(iGameModel.GameBoard()).CardImageSize().SetSpacialInformation(0, 0, iCardBitmapSize.iWidth, iCardBitmapSize.iHeight);

	return true;
}

void CGameCardRenderer::DrawCardStack(CFbsBitGc& aTargetGc,  const TRect& aTargetRect, const CGameCardStack& aGameCardStack)
{
	if (aGameCardStack.CardCount() > 0)
	{
		int i = 0;
		if (aGameCardStack.Type() != CGameCardStack::EInStack)
		{
			i = aGameCardStack.CardCount() - 2; // draw top card and card underneath top card only
			if (i < 0)
				i = 0;
		}	
		for (; i < aGameCardStack.CardCount(); ++i)
		{
			if (aGameCardStack.Card(i).RectIntersects(
				aTargetRect.iTl.iX, aTargetRect.iTl.iY, 
				aTargetRect.iBr.iX, aTargetRect.iBr.iY))
				DrawCard(aTargetGc, aGameCardStack.Card(i));
		}
		return;
	}

	int ix(0), iy(iCardBitmapSize.iHeight * 4);
	switch(aGameCardStack.Type())
	{
	case CGameCardStack::EOutStack:
		ix = iCardBitmapSize.iWidth * 10;
		break;
	case CGameCardStack::EInStack:
		ix = iCardBitmapSize.iWidth * 12;
		break;
	case CGameCardStack::EPlayer1InHand:
	case CGameCardStack::EPlayer2InHand:
		ix = iCardBitmapSize.iWidth * 11;
		break;
	case CGameCardStack::EPlayer1OutHand:
	case CGameCardStack::EPlayer2OutHand:
		ix = iCardBitmapSize.iWidth * 11;
		break;
	default:
		return;
	}

	int x,y,cx,cy;
	static_cast<const ISpacialInformation&>(aGameCardStack).SpacialInformation(x, y, cx, cy);

	aTargetGc.BitBltMasked(
		TPoint(x + (cx - iCardBitmapSize.iWidth) / 2, y + (cy - iCardBitmapSize.iHeight) / 2),
		iCardImagesBitmap, 
		TRect(TPoint(ix, iy), TSize(iCardBitmapSize.iWidth, iCardBitmapSize.iHeight)),
		iCardImagesBitmapMask, EFalse); 
}

void CGameCardRenderer::DrawCard(CFbsBitGc& aTargetGc, const CGameCard& aGameCard)
{
	int x,y,cx,cy;
	static_cast<const ISpacialInformation&>(aGameCard).SpacialInformation(x, y, cx, cy);
	aTargetGc.SetBrushColor(TRgb(0,0,0));
	aTargetGc.SetBrushStyle(CGraphicsContext::ENullBrush);
	aTargetGc.BitBltMasked(
		TPoint(x + (cx - iCardBitmapSize.iWidth) / 2, y + (cy - iCardBitmapSize.iHeight) / 2),
		iCardImagesBitmap,
		TRect(TPoint( 
		aGameCard.FaceDown() ? iCardBitmapSize.iWidth : aGameCard.Value() * iCardBitmapSize.iWidth,
		aGameCard.FaceDown() ? iCardBitmapSize.iHeight * 4 : aGameCard.Suit() * iCardBitmapSize.iHeight),
		TSize(iCardBitmapSize.iWidth, iCardBitmapSize.iHeight)),
		iCardImagesBitmapMask, EFalse);
}