// detail_memory.h
/* 
 * Copyright (c) 2008 Leigh Johnston.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this
 * software and its documentation for any purpose is hereby
 * granted, provided that the above copyright notice
 * appear in all copies and that both that copyright notice and
 * this permission notice appear in supporting documentation.
 * The author makes no representations about the
 * suitability of this software for any purpose. It is provided
 * "as is" without express or implied warranty.
 */

#ifndef LIB_DETAIL_MEMORY
#define LIB_DETAIL_MEMORY

#include <cstring>
#include "type.h"

namespace lib 
{
	namespace detail
	{
		template <typename T> inline
		void construct(void* mem, const T& object)
		{
			new (mem) T(object);
		}

		template <typename InIter, typename OutIter> inline
		OutIter uninitialized_copy_dispatch(InIter first, InIter last, OutIter result, false_type)
		{
			while (first != last)
				detail::construct(static_cast<void*>(&*result++), *first++);
			return result;
		}

		template <typename T> inline
		T* uninitialized_copy_dispatch(const T* first, const T* last, T* result, true_type)
		{
			memcpy(result, first, (last-first) * sizeof(T));
			result += (last-first);
			return result;
		}

		template <typename InIter, typename OutIter, typename T> inline
		OutIter uninitialized_copy(InIter first, InIter last, OutIter result, const T&)
		{
			return uninitialized_copy_dispatch(first, last, result, is_scalar<T>::type());
		}
	}
}

#endif // LIB_DETAIL_MEMORY