// observable.h
/* 
 * Copyright (c) 2008 Leigh Johnston.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this
 * software and its documentation for any purpose is hereby
 * granted, provided that the above copyright notice
 * appear in all copies and that both that copyright notice and
 * this permission notice appear in supporting documentation.
 * The author makes no representations about the
 * suitability of this software for any purpose. It is provided
 * "as is" without express or implied warranty.
 */

#ifndef LIB_OBSERVABLE
#define LIB_OBSERVABLE

#include <list>

namespace lib
{
	template <typename Observer>
	class observable
	{
	public:
		// types
		typedef Observer observer_type;
		typedef typename observer_type::notify_type notify_type;

	public:
		// operations
		virtual void add_observer(observer_type& aObserver)
		{
			iObservers.push_back(&aObserver);
		}
		virtual void remove_observer(observer_type& aObserver)
		{
			iObservers.remove(&aObserver);
		}

	protected:
		// implementation
		template <typename T>
		void notify_observers(notify_type aType, const T& aParameter)
		{
			notify_observers(aType, reinterpret_cast<const void*>(&aParameter));
		}
		template <typename T1, typename T2>
		void notify_observers(notify_type aType, const T1& aParameter, const T2& aParameter2)
		{
			notify_observers(aType, reinterpret_cast<const void*>(&aParameter), reinterpret_cast<const void*>(&aParameter2));
		}
		void notify_observers(notify_type aType, const void* aParameter = 0, const void* aParameter2 = 0)
		{
			for (observer_list::iterator i = iObservers.begin(); i != iObservers.end();)
				notify_observer(**i++, aType, aParameter, aParameter2);
		}
		template <typename T>
		void notify_observer(observer_type& aObserver, notify_type aType, T& aParameter)
		{
			notify_observer(aObserver, aType, reinterpret_cast<const void*>(&aParameter));
		}
		template <typename T1, typename T2>
		void notify_observer(observer_type& aObserver, notify_type aType, const T1& aParameter, const T2& aParameter2)
		{
			notify_observer(aObserver, aType, reinterpret_cast<const void*>(&aParameter), reinterpret_cast<const void*>(&aParameter2));
		}
		virtual void notify_observer(observer_type& aObserver, notify_type aType, const void* aParameter = 0, const void* aParameter2 = 0) = 0;

	protected:
		// attributes
		typedef std::list<observer_type*> observer_list;
		observer_list iObservers;
	};
}

#endif //LIB_OBSERVABLE
