// xml_bits.h - v1.0.3
/* 
 * Copyright (c) 2008 Leigh Johnston.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this
 * software and its documentation for any purpose is hereby
 * granted, provided that the above copyright notice
 * appear in all copies and that both that copyright notice and
 * this permission notice appear in supporting documentation.
 * The author makes no representations about the
 * suitability of this software for any purpose. It is provided
 * "as is" without express or implied warranty.
 */

#include <string>
#include <algorithm>
#include "string.h"
#include "vecarray.h"

namespace lib
{
	template <typename CharT>
	const typename xml_element<CharT>::string& xml_element<CharT>::attribute_value(const string& aAttributeName) const
	{
		static const string null;

		for (typename attribute_list::const_iterator i = iAttributes.begin(); i != iAttributes.end(); ++i)
			if (i->first == aAttributeName)
				return i->second;

		return null;
	}

	template <typename CharT>
	typename xml_element<CharT>::string& xml_element<CharT>::attribute_value(const string& aAttributeName)
	{
		static string null;
		
		for (typename attribute_list::iterator i = iAttributes.begin(); i != iAttributes.end(); ++i)
			if (i->first == aAttributeName)
				return i->second;

		null = string();
		return null;
	}

	template <typename CharT>
	const typename basic_xml<CharT>::predefined_entity basic_xml<CharT>::sPredefinedEntities[basic_xml<CharT>::PredefinedEntityCount] =
	{
		{ "amp", "&" },
		{ "lt", "<" },
		{ "gt", ">" },
		{ "apos", "\'" },
		{ "quot", "\"" },
	};
	template <>
	const basic_xml<wchar_t>::predefined_entity basic_xml<wchar_t>::sPredefinedEntities[basic_xml<wchar_t>::PredefinedEntityCount] =
	{
		{ L"amp", L"&" },
		{ L"lt", L"<" },
		{ L"gt", L">" },
		{ L"apos", L"\'" },
		{ L"quot", L"\"" },
	};

	template <typename CharT>
	basic_xml<CharT>::basic_xml() : iError(false), iIndentChar(sTabChar[0]), iIndentCount(1) 
	{
		for (std::size_t entityIndex = 0; entityIndex < PredefinedEntityCount; ++entityIndex)
			iEntities.push_back(std::make_pair(std::string(sPredefinedEntities[entityIndex].iFirst), std::string(sPredefinedEntities[entityIndex].iSecond)));
	}

	template <typename CharT>
	void basic_xml<CharT>::clear()
	{
		iRoot = element();
		iPool.clear();
		iError = false;
	}

	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sTabChar = "\t";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sSpaceChar = " ";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sLessThanChar = "<";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sGreaterThanChar = ">";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sEqualsChar = "=";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sForwardSlashChar = "/";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sAmpersandChar = "&";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sSemicolonChar = ";";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sHashChar = "#";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sHexChar = "x";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sQuoteChar = "\"";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sNameDelimeter = "<> /=\"\'";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sNameBadDelimeter = "<=\"\'";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sAttributeNameDelimeter = " =";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sAttributeValueDelimeter = "\"\'";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sTagDelimeter = "<>";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sElementTagStart = "<";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sElementTagEnd= ">";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sWhiteSpace = " \t\r\n";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sCommentStart = "!--";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sCommentEnd = "-->";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sDeclarationStart = "?";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sDeclarationEnd = "?>";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sEmptyTagWithAttributes = " />";
	template <typename CharT>
	const typename basic_xml<CharT>::string basic_xml<CharT>::sEmptyTag = "/>";

	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sTabChar = L"\t";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sSpaceChar = L" ";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sLessThanChar = L"<";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sGreaterThanChar = L">";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sEqualsChar = L"=";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sForwardSlashChar = L"/";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sAmpersandChar = L"&";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sSemicolonChar = L";";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sHashChar = L"#";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sHexChar = L"x";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sQuoteChar = L"\"";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sNameDelimeter = L"<> /=\"\'";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sNameBadDelimeter = L"<=\"\'";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sAttributeNameDelimeter = L" =";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sAttributeValueDelimeter = L"\"\'";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sTagDelimeter = L"<>";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sElementTagStart = L"<";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sElementTagEnd= L">";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sWhiteSpace = L" \t\r\n";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sCommentStart = L"!--";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sCommentEnd = L"-->";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sDeclarationStart = L"?";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sDeclarationEnd = L"?>";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sEmptyTagWithAttributes = L" />";
	template <>
	const basic_xml<wchar_t>::string basic_xml<wchar_t>::sEmptyTag = L"/>";

	template <typename CharT>
	typename basic_xml<CharT>::tag basic_xml<CharT>::next_tag(typename string::const_iterator aNext, typename string::const_iterator aDocumentEnd)
	{
		tag nextTag;
		nextTag.first = std::find_first_of(aNext, aDocumentEnd, sElementTagStart.begin(), sElementTagStart.end());
		if (nextTag.first != aDocumentEnd)
			++nextTag.first;
		nextTag.second = std::find_first_of(nextTag.first, aDocumentEnd, sTagDelimeter.begin(), sTagDelimeter.end());
		if (nextTag.second - nextTag.first >= 3 && std::equal(nextTag.first, nextTag.first+3, sCommentStart.begin()))
		{
			typename string::const_iterator next = std::find_first_of(nextTag.first+3, aDocumentEnd, sElementTagEnd.begin(), sElementTagEnd.end());
			if (next != aDocumentEnd)
				return next_tag(next+1, next+1);
			else
				return next_tag(aDocumentEnd, aDocumentEnd);
		}
		if (nextTag.second - nextTag.first >= 1 && std::equal(nextTag.first, nextTag.first+1, sDeclarationStart.begin()))
		{
			typename string::const_iterator next = std::find_first_of(nextTag.first+1, aDocumentEnd, sElementTagEnd.begin(), sElementTagEnd.end());
			if (next != aDocumentEnd)
				return next_tag(next+1, next+1);
			else
				return next_tag(aDocumentEnd, aDocumentEnd);
		}
		return nextTag;
	}

	template <typename CharT>
	typename basic_xml<CharT>::string::const_iterator basic_xml<CharT>::parse(element& aElement, const tag& aStartTag, typename string::const_iterator aDocumentEnd)
	{
		if (aStartTag.first >= aStartTag.second)
			return aDocumentEnd;

		/* get element name */
		vecarray<string, 1> elementName;
		typename string::const_iterator next = tokens(aStartTag.first, aStartTag.second, sNameDelimeter.begin(), sNameDelimeter.end(), elementName, 1, false);
		if (elementName.empty() || std::find_first_of(next-1, next, sNameBadDelimeter.begin(), sNameBadDelimeter.end()) != next)
			return aDocumentEnd;
		aElement.name() = elementName[0];

		/* get element attributes */
		while(next != aStartTag.second)
		{
			vecarray<string, 1> attributeName;
			next = tokens(next, aStartTag.second, sAttributeNameDelimeter.begin(), sAttributeNameDelimeter.end(), attributeName, 1);
			if (attributeName.empty() || *(next-1) != sEqualsChar[0])
				break;
			vecarray<string, 2> attributeValue;
			next = tokens(next, aStartTag.second, sAttributeValueDelimeter.begin(), sAttributeValueDelimeter.end(), attributeValue, 2, false);
			if (attributeValue.size() != 2)
			{
				iError = true;
				break;
			}
			std::pair<string, string> attribute = std::make_pair(attributeName[0], parse_entities(attributeValue[1]));
			aElement.attributes().push_back(attribute);
		}

		if (*(aStartTag.second-1) == sForwardSlashChar[0]) // empty tag
			return next+1;

		++next;

		/* get element content */
		while(next != aDocumentEnd)
		{
			vecarray<string, 1> contentPart;
			next = tokens(next, aDocumentEnd, sTagDelimeter.begin(), sTagDelimeter.end(), contentPart, 1, false);
			if (next == aDocumentEnd)
				return next;
			if (!contentPart.empty() && !contentPart[0].empty())
			{
				iPool.push_back(element());
				contentPart[0] = parse_entities(contentPart[0]);
				iPool.back().content() = contentPart[0];
				aElement.children().push_back(&iPool.back());
				aElement.content() += contentPart[0];
			}
			tag nextTag = next_tag(next-1, aDocumentEnd); 
			if (nextTag.first > nextTag.second)
				return next;
			if (nextTag.first == nextTag.second)
			{
				next= nextTag.first;
				continue;
			}
			if (*nextTag.first == sForwardSlashChar[0])
			{
				if (aElement.name() != string(nextTag.first+1, nextTag.second))
				{
					iError = true;
					return aDocumentEnd;
				}
				return nextTag.second+1;
			}
			iPool.push_back(element());
			aElement.children().push_back(&iPool.back());
			next = parse(iPool.back(), nextTag, aDocumentEnd);
		}

		return next;
	}

	template <typename CharT>
	bool basic_xml<CharT>::read(std::basic_istream<CharT>& aStream)
	{
		iError = false;

		string document;
		string line;
		while(std::getline(aStream, line))
		{
			typename string::size_type start = line.find_first_not_of(sWhiteSpace);
			if (start != string::npos)
				line.erase(0, start);
			typename string::size_type end = line.find_last_not_of(sWhiteSpace);
			if (end != string::npos)
				line.erase(end+1, line.size() - (end+1));
			if (document.size() != 0)
			{
				CharT chEnd = document[document.size()-1];
				if (line.size() != 0)
				{
					CharT chStart = line[0];
					if (chEnd != sGreaterThanChar[0] && chEnd != sSpaceChar[0] && chStart != sSpaceChar[0])
						document += sSpaceChar;
				}
			}
			document += line;
		}

		tag rootStartTag = next_tag(document.begin(), document.end());
		while(rootStartTag.first != document.end() && rootStartTag.first == rootStartTag.second)
			rootStartTag = next_tag(rootStartTag.first, document.end());
		
		if (rootStartTag.first == document.end() || rootStartTag.second == document.end())
				return false;

		parse(iRoot, rootStartTag, document.end());

		return true;
	}

	template <typename CharT>
	bool basic_xml<CharT>::write(std::basic_ostream<CharT>& aStream)
	{
		iError = false;

		write_element(aStream, iRoot, 0);
		return true;
	}

	template <typename CharT>
	void basic_xml<CharT>::set_indent(CharT aIndentChar, std::size_t aIndentCount)
	{
		iIndentChar = aIndentChar;
		iIndentCount = aIndentCount;
	}

	template <typename CharT>
	void basic_xml<CharT>::write_element(std::basic_ostream<CharT>& aStream, element& aElement, std::size_t aIndent)
	{
		if (aElement.name().length() != 0)
		{
			aStream << string(aIndent*iIndentCount, iIndentChar);
			aStream << sLessThanChar;
			aStream << aElement.name();
		}
		else
			aStream << generate_entities(aElement.content());

		if (aElement.attributes().size())
			for (typename attribute_list::iterator i = aElement.attributes().begin(); i != aElement.attributes().end(); ++i)
				aStream << sSpaceChar << i->first << sEqualsChar << sQuoteChar << generate_entities(i->second) << sQuoteChar;

		if (aElement.children().size())
		{
			aStream << sGreaterThanChar;
			bool justText = true;
			for (typename element_list::iterator i = aElement.children().begin(); i != aElement.children().end() && justText; ++i)
				if ((*i)->name().length() != 0)
					justText = false;
			if (!justText)
				aStream << std::endl;
			for (typename element_list::iterator i = aElement.children().begin(); i != aElement.children().end(); ++i)
				write_element(aStream, *(*i), aIndent+1);
			aStream << (!justText ? string(aIndent*iIndentCount, iIndentChar) : string()) << sLessThanChar << sForwardSlashChar << aElement.name() << sGreaterThanChar << std::endl;
		}
		else if (aElement.name().size() != 0)
			aStream << (aElement.attributes().size() ? sEmptyTagWithAttributes : sEmptyTag) << std::endl;
	}

	template <typename CharT>
	typename basic_xml<CharT>::element& basic_xml<CharT>::insert(element& aParent, typename element_list::iterator aPosition, const CharT* aName)
	{
		iPool.push_back(element());
		element& newElement = iPool.back();
		newElement.name() = aName;
		aParent.children().insert(aPosition, &newElement);
		return newElement;
	}

	template <typename CharT>
	typename basic_xml<CharT>::element& basic_xml<CharT>::append(element& aParent, const CharT* aName)
	{
		typename element_list::iterator endPos = aParent.children().end();
		return insert(aParent, endPos, aName);
	}
	
	template <typename CharT>
	void basic_xml<CharT>::erase(element& aParent, typename element_list::iterator aPosition)
	{
		for (typename element_list::iterator i = (*aPosition)->children().begin(); i != (*aPosition)->children().end(); ++i)
			erase(**aPosition, i);
		for (typename element_pool::iterator i = iPool.begin(); i != iPool.end(); ++i)
		{
			if (&(*i) == *aPosition)
			{
				iPool.erase(i);
				break;
			}
		}
		aParent.children().erase(aPosition);
	}
	
	template <typename CharT>
	typename basic_xml<CharT>::element_list::iterator basic_xml<CharT>::find(element& aParent, const CharT* aName)
	{
		for (typename element_list::iterator i = aParent.children().begin(); i != aParent.children().end(); ++i)
			if ((*i)->name() == aName)
				return i;
		return aParent.children().end();
	}

	template <typename CharT>
	typename basic_xml<CharT>::string basic_xml<CharT>::parse_entities(const string& aString)
	{
		string newString = aString;
		for(typename entity_list::const_iterator i = iEntities.begin(); i != iEntities.end(); ++i)
		{
			string placeholder;
			placeholder += sAmpersandChar;
			placeholder += i->first;
			placeholder += sSemicolonChar;
			typename string::size_type pos = 0;
			while((pos = newString.find(placeholder, pos)) != string::npos)
			{
				newString.replace(pos, placeholder.size(), i->second);
				pos += i->second.size();
			}
		}
	
		typename string::size_type pos = 0;
		while((pos = newString.find(sAmpersandChar, pos)) != string::npos)
		{
			typename string::size_type endPos = newString.find(sSemicolonChar, pos);
			if (endPos == string::npos)
			{
				iError = true;
				return aString;
			}
			if (newString[pos+1] == sHashChar[0])
			{
				string characterValue = newString.substr(pos+2, endPos - (pos + 2));
				string character;
				if (characterValue[0] != sHexChar[0])
					character += static_cast<CharT>(string_to_integer(characterValue, 10));
				else
				{
					characterValue.erase(0, 1);
					character += static_cast<CharT>(string_to_integer(characterValue, 16));
				}
				newString.replace(pos, (endPos - pos) + 1, character);
				++pos;
			}
			else
				newString.erase(pos, (endPos - pos) + 1);
		}

		return newString;
	}

	template <typename CharT>
	typename basic_xml<CharT>::string basic_xml<CharT>::generate_entities(const string& aString)
	{
		string newString = aString;
		for(typename entity_list::const_iterator i = iEntities.begin(); i != iEntities.end(); ++i)
		{
			string placeholder;
			placeholder += sAmpersandChar;
			placeholder += i->first;
			placeholder += sSemicolonChar;
			typename string::size_type pos = 0;
			while((pos = newString.find(i->second, pos)) != string::npos)
			{
				newString.replace(pos, i->second.size(), placeholder);
				pos += placeholder.size();
			}
		}
	
		return newString;
	}

} // namespace lib