// type.h
/* 
 * Copyright (c) 2008 Leigh Johnston.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this
 * software and its documentation for any purpose is hereby
 * granted, provided that the above copyright notice
 * appear in all copies and that both that copyright notice and
 * this permission notice appear in supporting documentation.
 * The author makes no representations about the
 * suitability of this software for any purpose. It is provided
 * "as is" without express or implied warranty.
 */

#ifndef LIB_TYPE
#define LIB_TYPE

namespace lib 
{
	template <typename T>
	inline unsigned_cast(T v) { return static_cast<T>(v); }
	template <>
	inline unsigned_cast<char>(char v) { return static_cast<unsigned char>(v); }
	template <>
	inline unsigned_cast<signed char>(signed char v) { return static_cast<unsigned char>(v); }
	template <>
	inline unsigned_cast<short>(short v) { return static_cast<unsigned short>(v); }
	template <>
	inline unsigned_cast<int>(int v) { return static_cast<unsigned int>(v); }
	template <>
	inline unsigned_cast<long>(long v) { return static_cast<unsigned long>(v); }

	template <typename T, typename Src>
	inline T enum_cast(Src v) { return static_cast<T>(reinterpret_cast<int>(v)); }

	template <typename T, T v>
	struct integral_constant 
	{ 
      static const T value;
      typedef T value_type;
      typedef integral_constant<T, v> type;
	};
	template <typename T, T v>
	const T integral_constant<T, v>::value = v;

	typedef integral_constant<bool, false> false_type;
	typedef integral_constant<bool, true> true_type;

	template <typename T>
	struct is_scalar : public false_type {};
	template <> struct is_scalar<bool> : public true_type {};
	template <> struct is_scalar<char> : public true_type {};
	template <> struct is_scalar<signed char> : public true_type {};
	template <> struct is_scalar<unsigned char> : public true_type {};
	template <> struct is_scalar<short> : public true_type {};
	template <> struct is_scalar<unsigned short> : public true_type {};
	template <> struct is_scalar<int> : public true_type {};
	template <> struct is_scalar<unsigned int> : public true_type {};
	template <> struct is_scalar<long> : public true_type {};
	template <> struct is_scalar<unsigned long> : public true_type {};
	template <> struct is_scalar<float> : public true_type {};
	template <> struct is_scalar<double> : public true_type {};
	template <> struct is_scalar<long double> : public true_type {};
	template <> struct is_scalar<__int64> : public true_type {};
	template <> struct is_scalar<void*> : public true_type {};
	template <> struct is_scalar<bool*> : public true_type {};
	template <> struct is_scalar<char*> : public true_type {};
	template <> struct is_scalar<signed char*> : public true_type {};
	template <> struct is_scalar<unsigned char*> : public true_type {};
	template <> struct is_scalar<short*> : public true_type {};
	template <> struct is_scalar<unsigned short*> : public true_type {};
	template <> struct is_scalar<int*> : public true_type {};
	template <> struct is_scalar<unsigned int*> : public true_type {};
	template <> struct is_scalar<long*> : public true_type {};
	template <> struct is_scalar<unsigned long*> : public true_type {};
	template <> struct is_scalar<float*> : public true_type {};
	template <> struct is_scalar<double*> : public true_type {};
	template <> struct is_scalar<long double*> : public true_type {};

	template <typename T>
	struct is_integral : public false_type {};
	template <> struct is_integral<bool> : public true_type {};
	template <> struct is_integral<char> : public true_type {};
	template <> struct is_integral<signed char> : public true_type {};
	template <> struct is_integral<unsigned char> : public true_type {};
	template <> struct is_integral<short> : public true_type {};
	template <> struct is_integral<unsigned short> : public true_type {};
	template <> struct is_integral<int> : public true_type {};
	template <> struct is_integral<unsigned int> : public true_type {};
	template <> struct is_integral<long> : public true_type {};
	template <> struct is_integral<unsigned long> : public true_type {};
	template <> struct is_integral<__int64> : public true_type {};
}

#endif // LIB_TYPE