// utility.h - v1.0
/* 
 * Copyright (c) 2008 Leigh Johnston.  All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this
 * software and its documentation for any purpose is hereby
 * granted, provided that the above copyright notice
 * appear in all copies and that both that copyright notice and
 * this permission notice appear in supporting documentation.
 * The author makes no representations about the
 * suitability of this software for any purpose. It is provided
 * "as is" without express or implied warranty.
 */

/* WARNING: The classes present here are not a substitute for any equivalent std:: 
 * classes available on your platform which you should be using instead.  They exist here
 * either for technical reasons or for when there is no standard library available.
  */

#ifndef LIB_UTILITY
#define LIB_UTILITY

namespace lib
{
	template <typename T1, typename T2>
	struct pair
	{
		typedef T1 first_type;
		typedef T2 second_type;
		T1 first;
		T2 second;
		pair() : first(T1()), second(T2()) {}
		pair(const T1& x, const T2& y) : first(x), second(y) {}
		template <typename U, typename V> pair(const pair<U, V>& p) : first(p.first), second(p.second) {}
		template <typename U, typename V> pair<T1, T2>& operator=(const pair<U, V>& p) { first = p.first; second = p.second; return *this; }
		pair operator-() const { return make_pair(-first, -second); }
	};

	template <typename T1, typename T2>
	inline pair<T1, T2> make_pair(T1 x, T2 y)
	{
		return pair<T1, T2>(x, y);
	}

	template <typename T1, typename T2>
	struct minmax : lib::pair<T1, T2>
	{
		minmax() {}
		minmax(const T1& x, const T2& y) : pair<T1, T2>(x, y) {}
		template <typename U, typename V> minmax(const minmax<U, V>& m) : pair<T1, T2>(m.first, m.second) {}
		template <typename U, typename V> minmax<T1, T2>& operator=(const minmax<U, V>& m) { first = m.first; second = m.second; return *this; }
		minmax operator-() const { minmax ret(-second, -first); return ret; }
	};

	template<typename T>
	struct exit_value
	{
		T& object;
		T value;
		exit_value(T& o, T v) : object(o), value(v) {}
		~exit_value() { object = value; }
	};
}

#endif // LIB_UTILITY